from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.user import UserDetailed
from mipac.types.admin import (
    IIndexStat,
    IModerationLog,
    IServerInfo,
    IServerInfoCpu,
    IServerInfoFs,
    IServerInfoMem,
    IServerInfoNet,
    IUserIP,
)
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class UserIP(AbstractModel):
    def __init__(self, user_ip: IUserIP) -> None:
        self.__user_ip: IUserIP = user_ip

    @property
    def ip(self) -> str:
        return self.__user_ip["ip"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__user_ip["created_at"])

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, UserIP) and self.ip == __value.ip

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class IndexStat:
    def __init__(self, index_stat: IIndexStat) -> None:
        self.__index_stat: IIndexStat = index_stat

    @property
    def schemaname(self) -> str:
        return self.__index_stat["schemaname"]

    @property
    def tablename(self) -> str:
        return self.__index_stat["tablename"]

    @property
    def indexname(self) -> str:
        return self.__index_stat["indexname"]

    @property
    def tablespace(self) -> str | None:
        return self.__index_stat["tablespace"]

    @property
    def indexdef(self) -> str:
        return self.__index_stat["indexdef"]


class ModerationLog(AbstractModel):
    def __init__(self, moderation_log: IModerationLog, *, client: ClientManager) -> None:
        self.__moderation_log: IModerationLog = moderation_log
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__moderation_log["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__moderation_log["created_at"])

    @property
    def type(self) -> str:
        return self.__moderation_log["type"]

    @property
    def info(self) -> dict:
        return self.__moderation_log["info"]

    @property
    def user_id(self) -> str:
        return self.__moderation_log["user_id"]

    @property
    def user(self) -> UserDetailed:
        return UserDetailed(self.__moderation_log["user"], client=self.__client)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, ModerationLog) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class ServerInfoCpu(AbstractModel):
    def __init__(self, server_info_cpu: IServerInfoCpu) -> None:
        self.__server_info_cpu = server_info_cpu

    @property
    def models(self) -> str:
        return self.__server_info_cpu["models"]

    @property
    def cores(self) -> int:
        return self.__server_info_cpu["cores"]


class ServerInfoMem(AbstractModel):
    def __init__(self, server_info_mem: IServerInfoMem) -> None:
        self.__server_info_mem = server_info_mem

    @property
    def total(self) -> int:
        return self.__server_info_mem["total"]


class ServerInfoFs(AbstractModel):
    def __init__(self, server_info_fs: IServerInfoFs) -> None:
        self.__server_info_fs = server_info_fs

    @property
    def total(self) -> int:
        return self.__server_info_fs["total"]

    @property
    def used(self) -> int:
        return self.__server_info_fs["used"]


class ServerInfoNet(AbstractModel):
    def __init__(self, server_info_net: IServerInfoNet) -> None:
        self.__server_info_net = server_info_net

    @property
    def interface(self) -> str:
        return self.__server_info_net["interface"]


class ServerInfo(AbstractModel):
    def __init__(self, server_info: IServerInfo) -> None:
        self.__server_info = server_info

    @property
    def machine(self) -> str:
        return self.__server_info["machine"]

    @property
    def os(self) -> str:
        return self.__server_info["os"]

    @property
    def node(self) -> str:
        return self.__server_info["node"]

    @property
    def psql(self) -> str:
        return self.__server_info["psql"]

    @property
    def cpu(self) -> ServerInfoCpu:
        return ServerInfoCpu(self.__server_info["cpu"])

    @property
    def mem(self) -> ServerInfoMem:
        return ServerInfoMem(self.__server_info["mem"])

    @property
    def fs(self) -> ServerInfoFs:
        return ServerInfoFs(self.__server_info["fs"])

    @property
    def net(self) -> ServerInfoNet:
        return ServerInfoNet(self.__server_info["net"])



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion, NotSupportVersionText, ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.channel import Channel
from mipac.models.lite.channel import ChannelLite
from mipac.types.channel import IChannel
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class ClientChannelActions(AbstractAction):
    def __init__(
        self, channel_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        self._channel_id: str | None = channel_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def favorite(self, channel_id: str | None = None):
        """
        Favorite a channel.

        Parameters
        ----------
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        bool
            True if success else False
        """

        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise Exception()

        res: bool = await self._session.request(
            Route("POST", "/api/channels/favorite"), auth=True, json={"channelId": channel_id}
        )

        return res

    async def unfavorite(self, channel_id: str | None = None):
        """
        Unfavorite a channel.

        Parameters
        ----------
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        bool
            True if success else False
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise Exception()

        res: bool = await self._session.request(
            Route("POST", "/api/channels/unfavorite"), auth=True, json={"channelId": channel_id}
        )

        return res

    async def follow(self, channel_id: str | None = None) -> bool:
        """
        Follow a channel.

        Parameters
        ----------
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        bool
            True if success else False
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise ParameterError("required channel_id")

        res: bool = await self._session.request(
            Route("POST", "/api/channels/follow"), auth=True, json={"channelId": channel_id}
        )

        return res

    async def unfollow(self, channel_id: str | None = None) -> bool:
        """
        Unfollow a channel.

        Parameters
        ----------
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        bool
            True if success else False
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise ParameterError("required channel_id")

        res: bool = await self._session.request(
            Route("POST", "/api/channels/unfollow"), auth=True, json={"channelId": channel_id}
        )

        return res

    async def update(
        self,
        name: str | None = None,
        description: str | None = None,
        banner_id: str | None = None,
        is_archived: bool | None = None,
        pinned_note_ids: list[str] | None = None,
        color: str | None = None,
        *,
        channel_id: str | None = None,
    ) -> Channel:
        """
        Update a channel.

        Parameters
        ----------
        name : str, optional, by default None
            Channel name
        description : str, optional, by default None
            Channel description
        banner_id : str, optional, by default None
            Channel banner id
        is_archived : bool, optional, by default None
            Channel is archived
        pinned_note_ids : list[str], optional, by default None
            Channel pinned note ids
        color : str, optional, by default None
            Channel color
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        Channel
            Channel
        """

        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise ParameterError("required channel_id")

        body = {
            "channelId": channel_id,
            "name": name,
            "description": description,
            "bannerId": banner_id,
            "isArchived": is_archived,
            "pinnedNoteIds": pinned_note_ids,
            "color": color,
        }
        res: IChannel = await self._session.request(
            Route("POST", "/api/channels/update"), auth=True, json=body
        )

        return Channel(res, client=self._client)

    async def archive(self, channel_id: str | None = None) -> Channel:
        """
        Archive a channel.

        Parameters
        ----------
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        Channel
            Channel
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise ParameterError("required channel_id")

        res = await self.update(channel_id=channel_id, is_archived=True)

        return res

    async def unarchive(self, channel_id: str | None = None) -> Channel:
        """
        Unarchive a channel.

        Parameters
        ----------
        channel_id : str, optional, by default None
            Channel id

        Returns
        -------
        Channel
            Channel
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        channel_id = self._channel_id or channel_id
        if channel_id is None:
            raise ParameterError("required channel_id")

        res = await self.update(channel_id=channel_id, is_archived=False)

        return res


class ChannelActions(ClientChannelActions):
    def __init__(
        self, channel_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        super().__init__(channel_id=channel_id, session=session, client=client)

    async def create(
        self,
        name: str,
        description: str | None = None,
        banner_id: str | None = None,
        color: str = "#000",
    ) -> ChannelLite:
        """
        Create a channel.

        Parameters
        ----------
        name : str
            Channel name.
        description : str, optional, by default None
            Channel description
        banner_id : str, optional, by default None
            Channel banner id
        color : str, optional, by default '#000'
            Channel color

        Returns
        -------
        Channel
            ChannelLite
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        body = {"name": name, "description": description, "bannerId": banner_id, "color": color}
        res: IChannel = await self._session.request(
            Route("POST", "/api/channels/create"), auth=True, json=body
        )

        return ChannelLite(res, client=self._client)

    async def get_featured(self) -> list[Channel]:
        """
        Get featured channels.

        Returns
        -------
        list[Channel]
            Channel
        """

        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        res: list[IChannel] = await self._session.request(
            Route("POST", "/api/channels/featured"), auth=True
        )
        return [Channel(i, client=self._client) for i in res]

    async def get_followed(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[Channel, None]:
        """
        Get followed channels.

        Parameters
        ----------
        since_id : str, optional, by default None
            Since id
        until_id : str, optional, by default None
            Until id
        limit : int, optional, by default 10
            Limit
        get_all : bool, optional, by default False
            Get all channels flag

        Yields
        -------
        AsyncGenerator[Channel, None]
            Channel
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        body = {"sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[IChannel](
            self._session, Route("POST", "/api/channels/followed"), auth=True, json=body
        )

        while True:
            raw_channels = await pagination.next()
            for raw_channel in raw_channels:
                yield Channel(raw_channel, client=self._client)

            if get_all is False or pagination.is_final:
                break

    async def get_owned(
        self, since_id: str | None = None, until_id: str | None = None, *, get_all: bool = False
    ) -> AsyncGenerator[Channel, None]:
        """
        Get owned channels.

        Parameters
        ----------
        since_id : str, optional, by default None
            Since id
        until_id : str, optional, by default None
            Until id
        get_all : bool, optional, by default False
            Get all channels flag

        Yields
        ------
        AsyncGenerator[Channel, None]
            Channel
        """

        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        body = {"sinceId": since_id, "untilId": until_id}

        pagination = Pagination[IChannel](
            self._session, Route("POST", "/api/channels/owned"), auth=True, json=body
        )

        while True:
            raw_channels = await pagination.next()
            for raw_channel in raw_channels:
                yield Channel(raw_channel, client=self._client)

            if get_all is False or pagination.is_final:
                break

    async def get(self, channel_id: str) -> Channel:
        """
        Get a channel.

        Parameters
        ----------
        channel_id : str
            Channel id

        Returns
        -------
        Channel
            Channel
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        res: IChannel = await self._session.request(
            Route("POST", "/api/channels/show"), auth=True, json={"channelId": channel_id}
        )
        return Channel(res, client=self._client)

    async def get_my_favorite(self) -> list[Channel]:
        """
        Get my favorite channels.

        Returns
        -------
        list[Channel]
            Channel
        """
        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        res: list[IChannel] = await self._session.request(
            Route("POST", "/api/channels/my-favorites"), auth=True
        )
        return [Channel(i, client=self._client) for i in res]

    async def search(
        self,
        query: str,
        type: Literal["nameAndDescription", "nameOnly"] = "nameAndDescription",
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 5,
        get_all: bool = False,
    ) -> AsyncGenerator[Channel, None]:
        """
        Search channels.

        Parameters
        ----------
        query : str
            Search query
        type : Literal['nameAndDescription', 'nameOnly'], optional, by default 'nameAndDescription'
            Search type
        since_id : str, optional, by default None
            Since id
        until_id : str, optional, by default None
            Until id
        limit : int, optional, by default 5
            Limit
        get_all : bool, optional, by default False
            Get all channels flag

        Yields
        ------
        AsyncGenerator[Channel, None]
            Channel
        """

        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        body = {
            "query": query,
            "type": type,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
        }

        pagination = Pagination[IChannel](
            self._session, Route("POST", "/api/channels/search"), auth=True, json=body
        )

        while True:
            raw_channels = await pagination.next()
            for raw_channel in raw_channels:
                yield Channel(raw_channel, client=self._client)

            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import Route

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager


class AdminModeratorActions(AbstractAction):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__user_id: str | None = user_id

    async def add(self, user_id: str | None = None) -> bool:
        """
        Add a user as a moderator

        Parameters
        ----------
        user_id : str | None, default=None
            ユーザーのID

        Returns
        -------
        bool
            成功したか否か
        """

        user_id = user_id or self.__user_id
        data = {"userId": user_id}
        res = await self.__session.request(
            Route("POST", "/api/admin/moderators/add"),
            json=data,
            auth=True,
            lower=True,
        )
        return bool(res)

    async def remove(self, user_id: str | None = None) -> bool:
        """
        Unmoderate a user

        Parameters
        ----------
        user_id : str | None, default=None
            ユーザーのID

        Returns
        -------
        bool
            成功したか否か
        """
        user_id = user_id or self.__user_id
        data = {"userId": user_id}
        res = await self.__session.request(
            Route("POST", "/api/admin/moderators/remove"),
            json=data,
            auth=True,
            lower=True,
        )
        return bool(res)



import re
from datetime import datetime
from typing import Any, Mapping


def snake_to_camel(snake_str: str, replace_list: dict[str, str]) -> str:
    components: list[str] = snake_str.split("_")
    for i in range(len(components)):
        if components[i] in replace_list:
            components[i] = replace_list[components[i]]
    return components[0] + "".join(x.title() for x in components[1:])


def convert_dict_keys_to_camel(
    data: Mapping[Any, Any], replace_list: dict[str, str] | None = None
) -> Mapping[Any, Any]:
    if replace_list is None:
        replace_list = {}
    new_dict = {}
    for key, value in data.items():
        new_key = snake_to_camel(key, replace_list)
        new_dict[new_key] = value
    return new_dict


def str_to_datetime(data: str, format: str = "%Y-%m-%dT%H:%M:%S.%fZ") -> datetime:
    """
    Parameters
    ----------
    data : str
        datetimeに変更したい文字列
    format : str
        dataのフォーマット

    Returns
    -------
    datetime
        変換後のデータ
    """
    return datetime.strptime(data, format)


def remove_list_empty(data: list[Any]) -> list[Any]:
    """
    Parameters
    ----------
    data: dict
        空のkeyを削除したいdict

    Returns
    -------
    dict[str, Any]
        空のkeyがなくなったdict
    """
    return [k for k in data if k]


def remove_dict_empty(data: dict[str, Any]) -> dict[str, Any]:
    """
    Parameters
    ----------
    data: dict
        空のkeyを削除したいdict

    Returns
    -------
    _data: dict
        空のkeyがなくなったdict
    """
    _data = {}
    _data = {k: v for k, v in data.items() if v is not None}
    return _data


def upper_to_lower(
    data: dict[str, Any],
    field: dict[str, Any] | None = None,
    nest: bool = True,
    replace_list: dict[str, Any] | None = None,
) -> dict[str, Any]:
    """
    Parameters
    ----------
    data: dict
        小文字にしたいkeyがあるdict
    field: dict, default=None
        謎
    nest: bool, default=True
        ネストされたdictのkeyも小文字にするか否か
    replace_list: dict, default=None
        dictのkey名を特定の物に置き換える

    Returns
    -------
    field : dict
        小文字になった, key名が変更されたdict
    """
    if data is None:
        return {}
    if replace_list is None:
        replace_list = {}

    if field is None:
        field = {}
    for attr in data:
        pattern = re.compile("[A-Z]")
        large = [i.group().lower() for i in pattern.finditer(attr)]
        result = [None] * (len(large + pattern.split(attr)))
        result[::2] = pattern.split(attr)
        result[1::2] = ["_" + i.lower() for i in large]
        default_key = "".join(result)
        if replace_list.get(attr):
            default_key = default_key.replace(attr, replace_list.get(attr))
        field[default_key] = data[attr]
        if isinstance(field[default_key], dict) and nest:
            field[default_key] = upper_to_lower(field[default_key])
        elif isinstance(field[default_key], list) and nest:
            field[default_key] = [
                upper_to_lower(i) if isinstance(i, dict) else i for i in field[default_key]
            ]
    return field


def str_lower(text: str):
    pattern = re.compile("[A-Z]")
    large = [i.group().lower() for i in pattern.finditer(text)]
    result = [None] * (len(large + pattern.split(text)))
    result[::2] = pattern.split(text)
    result[1::2] = ["_" + i.lower() for i in large]
    return "".join(result)


def bool_to_string(boolean: bool) -> str:
    """
    boolを小文字にして文字列として返します

    Parameters
    ----------
    boolean : bool
        変更したいbool値
    Returns
    -------
    true or false: str
        小文字になったbool文字列
    """
    return "true" if boolean else "false"



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.chart import ActiveUsersChart, DriveChart

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ChartActions(AbstractAction):
    def __init__(self, session: HTTPClient, client: ClientManager):
        self.__session = session
        self.__client = client

    async def get_active_user(
        self, span: str = "day", limit: int = 30, offset: int = 0
    ) -> ActiveUsersChart:
        data = {"span": span, "limit": limit, "offset": offset}
        data = await self.__session.request(
            Route("POST", "/api/charts/active-users"),
            json=data,
            auth=True,
            lower=True,
        )
        return ActiveUsersChart(data)

    async def get_drive(self, span: str = "day", limit: int = 30, offset: int = 0) -> DriveChart:
        data = {"span": span, "limit": limit, "offset": offset}
        data = await self.__session.request(
            Route("POST", "/api/charts/drive"),
            json=data,
            auth=True,
            lower=True,
        )
        return DriveChart(data)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.antenna import AntennaActions, ClientAntennaActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientAntennaManager(AbstractManager):
    def __init__(self, *, antenna_id: str, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__antenna_id: str = antenna_id

    @property
    def action(self) -> ClientAntennaActions:
        return ClientAntennaActions(
            antenna_id=self.__antenna_id, session=self.__session, client=self.__client
        )


class AntennaManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AntennaActions:
        return AntennaActions(session=self.__session, client=self.__client)

    def _create_client_antenna_manager(self, antenna_id: str) -> ClientAntennaManager:
        return ClientAntennaManager(
            antenna_id=antenna_id, session=self.__session, client=self.__client
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from mipac.abstract.model import AbstractModel

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.drive import ClientFileManager, ClientFolderManager
    from mipac.types import FolderPayload, IDriveFile, IFileProperties

__all__ = ["FileProperties", "File", "Folder"]


class FileProperties(AbstractModel):
    def __init__(self, properties: IFileProperties) -> None:
        self.__properties: IFileProperties = properties

    @property
    def width(self) -> int | None:
        return self.__properties["width"]

    @property
    def height(self) -> int:
        return self.__properties["height"]

    @property
    def avg_color(self) -> str | None:
        return self.__properties["avg_color"]


class Folder(AbstractModel):
    def __init__(self, folder: FolderPayload, client: ClientManager):
        self.__folder: FolderPayload = folder
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """フォルダのID"""
        return self.__folder["id"]

    @property
    def created_at(self) -> str:  # TODO: 型
        """フォルダの作成日時"""
        return self.__folder["created_at"]

    @property
    def name(self) -> str:
        """フォルダ名"""
        return self.__folder["name"]

    @property
    def folders_count(self) -> int:
        """フォルダ内のフォルダ数"""
        return self.__folder["folders_count"]

    @property
    def files_count(self) -> int:
        """フォルダ内のファイル数"""
        return self.__folder["files_count"]

    @property
    def parent_id(self) -> str:
        return self.__folder["parent_id"]

    @property
    def parent(self) -> dict[str, Any]:
        return self.__folder["parent"]

    @property
    def api(self) -> ClientFolderManager:
        return self.__client.drive._get_client_folder_instance(folder_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Folder) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class File(AbstractModel):
    def __init__(self, file: IDriveFile, *, client: ClientManager):
        self.__file: IDriveFile = file
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__file["id"]

    @property
    def created_at(self):
        return self.__file["created_at"]

    @property
    def is_sensitive(self) -> bool:
        return self.__file["is_sensitive"]

    @property
    def name(self) -> str:
        return self.__file["name"]

    @property
    def thumbnail_url(self) -> str:
        return self.__file["thumbnail_url"]

    @property
    def url(self) -> str:
        return self.__file["url"]

    @property
    def type(self) -> str:
        return self.__file["type"]

    @property
    def size(self) -> int:
        return self.__file["size"]

    @property
    def md5(self) -> str:
        return self.__file["md5"]

    @property
    def blurhash(self) -> str:
        return self.__file["blurhash"]

    @property
    def properties(self) -> FileProperties:
        return FileProperties(self.__file["properties"])

    @property
    def api(self) -> ClientFileManager:
        return self.__client.drive._get_client_file_instance(file_id=self.id, url=self.url)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, File) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from .channel import *
from .chat import *
from .drive import *
from .emoji import *
from .follow import *
from .instance import *
from .lite import *
from .mute import *
from .note import *
from .notification import *
from .poll import *
from .reaction import *
from .user import *



from typing import TypedDict

__all__ = (
    "IActiveUsersChart",
    "IDriveLocalChart",
    "IDriveRemoteChart",
    "IDriveChart",
)


class IActiveUsersChart(TypedDict):
    read_write: list[int]
    read: list[int]
    write: list[int]
    registered_within_week: list[int]
    registered_within_month: list[int]
    registered_within_year: list[int]
    registered_outside_week: list[int]
    registered_outside_month: list[int]
    registered_outside_year: list[int]


class IDriveLocalChart(TypedDict):
    total_count: list[int]
    total_size: list[int]
    inc_count: list[int]
    inc_size: list[int]
    dec_count: list[int]
    dec_size: list[int]


class IDriveRemoteChart(TypedDict):
    total_count: list[int]
    total_size: list[int]
    inc_count: list[int]
    inc_size: list[int]
    dec_count: list[int]
    dec_size: list[int]


class IDriveChart(TypedDict):
    local: IDriveLocalChart
    remote: IDriveRemoteChart



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.actions.client import ClientActions
from mipac.http import HTTPClient
from mipac.manager.admins.admin import AdminManager
from mipac.manager.antenna import AntennaManager
from mipac.manager.channel import ChannelManager
from mipac.manager.chart import ChartManager
from mipac.manager.chat import ChatManager
from mipac.manager.clip import ClipManager
from mipac.manager.drive import DriveManager
from mipac.manager.emoji import EmojiManager
from mipac.manager.follow import FollowManager, FollowRequestManager
from mipac.manager.my import MyManager
from mipac.manager.note import NoteManager
from mipac.manager.role import RoleManager
from mipac.manager.user import UserManager

if TYPE_CHECKING:
    from mipac.config import Config
    from mipac.models.lite.user import LiteUser
    from mipac.models.user import UserDetailed


__all__ = ("ClientManager",)


class ClientManager:
    def __init__(self, session: HTTPClient, config: Config):
        self.__session: HTTPClient = session
        self.i = MyManager(session=session, client=self)
        self.note: NoteManager = NoteManager(session=session, client=self)
        self.chat: ChatManager = ChatManager(session=session, client=self)
        self.user: UserManager = UserManager(session=session, client=self)
        self.admin: AdminManager = AdminManager(session=session, client=self)
        self.drive: DriveManager = DriveManager(session=session, client=self)
        self.chart: ChartManager = ChartManager(session=session, client=self)
        self.channel: ChannelManager = ChannelManager(session=session, client=self)
        self.follow: FollowManager = FollowManager(
            session=session,
            client=self,
        )
        self.follow_request: FollowRequestManager = FollowRequestManager(
            session=session,
            client=self,
        )
        self.clip: ClipManager = ClipManager(session=session, client=self)
        self.emoji: EmojiManager = EmojiManager(session=session, client=self)
        self.antenna: AntennaManager = AntennaManager(session=session, client=self)
        self.role: RoleManager = RoleManager(session=session, client=self)
        self._config: Config = config

    @property
    def action(self) -> ClientActions:
        return ClientActions(session=self.__session, client=self)

    def _create_user_instance(self, user: LiteUser) -> UserManager:
        return UserManager(session=self.__session, client=self, user=user)

    def _create_note_instance(self, note_id: str) -> NoteManager:
        return NoteManager(note_id, session=self.__session, client=self)

    def _create_channel_instance(self, channel_id: str) -> ChannelManager:
        return ChannelManager(channel_id=channel_id, session=self.__session, client=self)

    async def get_me(self) -> UserDetailed:
        return await self.user.action.get_me()



from __future__ import annotations

from typing import TYPE_CHECKING, Literal

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.ad import AdminAdvertisingActions, AdminAdvertisingModelActions
from mipac.http import HTTPClient, Route

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ("AdminAdvertisingManager", "AdminAdvertisingModelManager")


class AdminAdvertisingModelManager(AbstractManager):
    def __init__(self, ad_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__ad_id: str | None = ad_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminAdvertisingModelActions:
        return AdminAdvertisingModelActions(
            ad_id=self.__ad_id, session=self.__session, client=self.__client
        )


class AdminAdvertisingManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminAdvertisingActions:
        return AdminAdvertisingActions(session=self.__session, client=self.__client)

    async def create(
        self,
        url: str,
        memo: str,
        place: str,
        priority: Literal["high", "middle", "low"],
        ratio: str,
        expires_at: int,
        image_url: str,
    ):
        data = {
            "url": url,
            "memo": memo,
            "place": place,
            "priority": priority,
            "ratio": ratio,
            "expires_at": expires_at,
            "image_url": image_url,
        }
        return await self.__session.request(
            Route("POST", "/api/admin/ad/create"),
            json=data,
            auth=True,
            lower=True,
        )



from typing import Optional, TypedDict

__all__ = ("EmojiPayload", "ICustomEmojiLite", "ICustomEmoji")


class ICustomEmojiLiteRequired(TypedDict):
    name: str


class ICustomEmojiLite(ICustomEmojiLiteRequired, total=False):
    url: str


class ICustomEmoji(ICustomEmojiLite):
    id: str
    category: str
    aliases: list[str]
    host: str | None
    license: str | None  # v13 only


class EmojiPayload(TypedDict):
    id: str | None
    aliases: Optional[list[str]]
    name: str | None
    category: str | None
    host: str | None
    url: str | None
    license: str | None  # v13 only



"""
WebSocketで来る通知の型を定義しています

NfはNotificationの略です
"""

from __future__ import annotations

from typing import TYPE_CHECKING, TypedDict

if TYPE_CHECKING:
    from mipac.types.note import INote
    from mipac.types.user import ILiteUser


class INotification(TypedDict):
    id: str
    type: str
    created_at: str
    is_read: bool


class IUserNf(INotification):
    user: ILiteUser
    user_id: str


class INoteNf(INotification):
    user: ILiteUser
    user_id: str
    note: INote


class IPollEndNf(INotification):
    note: INote


class IReactionNf(INotification):
    reaction: str
    user: ILiteUser
    user_id: str
    note: INote


class IAchievementNf(INotification):
    achievement: str



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.lite.emoji import PartialCustomEmoji
from mipac.types.note import INoteUpdated, INoteUpdatedReaction

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class PartialReaction(AbstractModel):
    def __init__(
        self, reaction: INoteUpdated[INoteUpdatedReaction], *, client: ClientManager
    ) -> None:
        self.__reaction: INoteUpdated[INoteUpdatedReaction] = reaction
        self.__client: ClientManager = client

    @property
    def reaction(self) -> str:
        return self.__reaction["body"]["body"]["reaction"]

    @property
    def emoji(self) -> PartialCustomEmoji:
        return PartialCustomEmoji(self.__reaction["body"]["body"]["emoji"], client=self.__client)

    @property
    def user_id(self) -> str:
        return self.__reaction["body"]["body"]["user_id"]



from ._version import get_versions

__title__ = "mipac"
__author__ = "yupix"
__license__ = "MIT"
__copyright__ = "Copyright 2022-present yupix"
__author_email__ = "yupi0982@outlook.jp"
__version__ = get_versions()["version"]
del get_versions

__path__ = __import__("pkgutil").extend_path(__path__, __name__)

from . import _version
from .abstract import *  # noqa: F403, F401
from .actions import *
from .file import *
from .manager import *
from .models import *  # noqa: F403, F401
from .types import *

__version__ = _version.get_versions()["version"]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.emoji import CustomEmoji
from mipac.types.ads import IAds
from mipac.types.meta import ICPU, ILiteMeta, IMetaAnnouncement, IMetaCommon

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class CPU(AbstractModel):
    def __init__(self, cpu: ICPU) -> None:
        self.__cpu: ICPU = cpu

    @property
    def cores(self) -> int:
        return self.__cpu["cores"]

    @property
    def model(self) -> str:
        return self.__cpu["model"]


class MetaCommon(AbstractModel):
    def __init__(self, meta_common: IMetaCommon, *, client: ClientManager) -> None:
        self.__meta_common: IMetaCommon = meta_common
        self.__client = client

    @property
    def cache_remote_files(self) -> bool:
        return self.__meta_common["cache_remote_files"]

    @property
    def enable_hcaptch(self) -> bool:
        return self.__meta_common["enable_hcaptch"]

    @property
    def hcaptcha_site_key(self) -> str | None:
        return self.__meta_common["hcaptcha_site_key"]

    @property
    def enable_recaptcha(self) -> bool:
        return self.__meta_common["enable_recaptcha"]

    @property
    def recaptcha_site_key(self) -> str:
        return self.__meta_common["recaptcha_site_key"]

    @property
    def sw_publickey(self) -> str | None:
        return self.__meta_common["sw_publickey"]

    @property
    def banner_url(self) -> str | None:
        return self.__meta_common["banner_url"]

    @property
    def error_image_url(self) -> str | None:
        return self.__meta_common["error_image_url"]

    @property
    def icon_url(self) -> str | None:
        return self.__meta_common["icon_url"]

    @property
    def max_note_text_length(self) -> int:
        return self.__meta_common["max_note_text_length"]

    @property
    def enable_email(self) -> bool:
        return self.__meta_common["enable_email"]

    @property
    def enable_twitter_integration(self) -> bool:
        return self.__meta_common["enable_twitter_integration"]

    @property
    def enable_github_integration(self) -> bool:
        return self.__meta_common["enable_github_integration"]

    @property
    def enable_discord_integration(self) -> bool:
        return self.__meta_common["enable_discord_integration"]

    @property
    def enable_service_worker(self) -> bool:
        return self.__meta_common["enable_service_worker"]

    @property
    def proxy_account_name(self) -> str | None:
        return self.__meta_common["proxy_account_name"]

    @property
    def use_star_for_reaction_fallback(self) -> bool:
        return self.__meta_common.get("use_star_for_reaction_fallback", False)

    @property
    def object_storage_s3_force_path_style(self) -> bool:
        """
        objectStorageでs3ForcePathStyleを使用するかどうか
        注意: v11の管理者のみ取得できます

        Returns
        -------
        bool
            有効かどうか
        """
        return self.__meta_common.get("object_storage_s3_force_path_style", False)

    # v12 only

    @property
    def ads(self) -> list[IAds] | None:  # TODO: モデルに
        return self.__meta_common.get("ads")

    @property
    def translator_available(self) -> bool:
        return self.__meta_common.get("translator_available", False)

    @property
    def email_required_for_signup(self) -> bool:
        return self.__meta_common.get("email_required_for_signup", False)

    @property
    def mascot_image_url(self) -> str | None:
        return self.__meta_common.get("mascot_image_url")

    # v12とv11の共通情報

    @property
    def emojis(self) -> list[CustomEmoji]:
        return (
            [CustomEmoji(i, client=self.__client) for i in self.__meta_common["emojis"]]
            if "emojis" in self.__meta_common
            else []
        )


class LiteMeta(MetaCommon):
    def __init__(self, meta: ILiteMeta, *, client: ClientManager) -> None:
        super().__init__(meta, client=client)

        self.__lite_meta: ILiteMeta = meta

    @property
    def description(self) -> str | None:
        return self.__lite_meta["description"]

    @property
    def disable_registration(self) -> bool:
        return self.__lite_meta["disable_registration"]

    @property
    def feedback_url(self) -> str:
        return self.__lite_meta["feedback_url"]

    @property
    def maintainer_email(self) -> str | None:
        return self.__lite_meta["maintainer_email"]

    @property
    def maintainer_name(self) -> str | None:
        return self.__lite_meta["maintainer_name"]

    @property
    def name(self) -> str | None:
        return self.__lite_meta["name"]

    @property
    def repository_url(self) -> str:
        return self.__lite_meta["repository_url"]

    @property
    def tos_url(self) -> str | None:
        return self.__lite_meta["tos_url"]

    @property
    def uri(self) -> str:
        return self.__lite_meta["uri"]

    @property
    def version(self) -> str:
        return self.__lite_meta["version"]

    # v12のMeta

    @property
    def background_image_url(self) -> str | None:
        return self.__lite_meta.get("background_image_url")

    @property
    def default_dark_theme(self) -> str | None:
        return self.__lite_meta.get("default_dark_theme")

    @property
    def default_light_theme(self) -> str | None:
        return self.__lite_meta.get("default_light_theme")

    @property
    def logo_image_url(self) -> str | None:
        return self.__lite_meta.get("logo_image_url")

    @property
    def theme_color(self) -> str | None:
        return self.__lite_meta.get("theme_color")

    # v11のMeta

    @property
    def announcements(self) -> list[IMetaAnnouncement] | None:  # TODO: 型確認
        return self.__lite_meta.get("announcements", None)

    @property
    def cpu(self) -> CPU | None:
        return CPU(self.__lite_meta["cpu"]) if "cpu" in self.__lite_meta else None

    @property
    def disable_local_timeline(self) -> bool:
        return self.__lite_meta.get("disable_local_timeline", False)

    @property
    def disable_global_timeline(self) -> bool:
        return self.__lite_meta.get("disable_global_timeline", False)

    @property
    def enable_emoji_reaction(self) -> bool:
        return self.__lite_meta.get("enable_emoji_reaction", False)

    @property
    def machine(self) -> str | None:
        return self.__lite_meta.get("machine")

    @property
    def node(self) -> str | None:
        return self.__lite_meta.get("node")

    @property
    def os(self) -> str | None:
        return self.__lite_meta.get("os")

    @property
    def proxy_account(self) -> str | None:
        return self.__lite_meta.get("proxy_account")

    @property
    def proxy_remote_files(self) -> bool:
        return self.__lite_meta.get("proxy_remote_files", False)

    @property
    def psql(self) -> str | None:
        return self.__lite_meta.get("psql")

    @property
    def redis(self) -> str | None:
        return self.__lite_meta.get("redis")

    @property
    def secure(self) -> bool:
        return self.__lite_meta.get("secure", False)

    @property
    def summaly_proxy(self) -> str | None:
        return self.__lite_meta.get("summaly_proxy")

    @property
    def tos_text_url(self) -> str | None:
        return self.__lite_meta.get("tos_text_url")

    @property
    def turnstile_secret_key(self) -> str | None:
        return self.__lite_meta.get("turnstile_secret_key")



from __future__ import annotations

from typing import TYPE_CHECKING, List, Literal, TypeGuard, TypedDict, Any


if TYPE_CHECKING:
    from mipac.types.announcement import IAnnouncement
    from mipac.types.emoji import ICustomEmojiLite
    from mipac.types.instance import IInstanceLite
    from mipac.types.note import INote
    from mipac.types.page import IPage
__all__ = (
    "IFollowRequest",
    "IUserRequired",
    "ILiteUser",
    "IUserDetailed",
    "IUserDetailedField",
    "IAchievement",
    "IBlockingUser",
)


class IBlockingUser(TypedDict):
    id: str
    created_at: str
    blockee_id: str
    blockee: IUserDetailed


class ISignin(TypedDict):
    id: str
    created_at: str
    user_id: str
    ip: str
    headers: dict[str, str]
    success: bool


class IAchievement(TypedDict):
    name: str
    unlocked_at: int


class IBadgeRole(TypedDict):
    name: str
    icon_url: str | None
    display_order: int


class IUserRole(IBadgeRole):
    id: str
    color: str | None
    description: str
    is_moderator: bool
    is_administrator: bool


class IUserRequired(TypedDict):
    id: str
    username: str
    name: str
    online_status: Literal["online", "active", "offline", "unknown"]
    avatar_url: str
    avatar_blurhash: str


class ILiteUser(IUserRequired, total=False):
    host: str
    instance: IInstanceLite
    emojis: list[ICustomEmojiLite]
    avatar_color: str
    badge_roles: list[IBadgeRole]  # v13なら絶対あるはずだけど、他のにはロールそのものが無いので


class IUserDetailedField(TypedDict):
    name: str
    value: str


class IUserDetailedRequired(ILiteUser):
    fields: list[IUserDetailedField]
    followers_count: int
    following_count: int
    has_pending_follow_request_from_you: bool
    has_pending_follow_request_to_you: bool
    is_admin: bool
    is_blocked: bool
    is_blocking: bool
    is_bot: bool
    is_cat: bool
    is_followed: bool
    is_following: bool
    is_locked: bool
    is_moderator: bool
    is_muted: bool
    is_silenced: bool
    is_suspended: bool
    public_reactions: bool
    security_keys: bool
    two_factor_enabled: bool
    notes_count: int
    pinned_note_ids: list[str]
    pinned_notes: List[INote]


class IUserDetailed(IUserDetailedRequired, total=False):
    achievements: List[IAchievement]
    banner_blurhash: str
    banner_color: str
    banner_url: str
    birthday: str
    created_at: str
    description: str
    ff_visibility: Literal["public", "followers", "private"]
    lang: str
    last_fetched_at: str
    location: str
    logged_in_days: int
    pinned_page: IPage
    pinned_page_id: str
    updated_at: str
    uri: str
    url: str
    roles: list[IUserRole]
    memo: str | None
    moderation_note: str  # Noneではなく空の文字列


class IMeDetailed(IUserDetailed):
    avatar_id: str
    banner_id: str
    auto_accept_followed: bool
    always_mark_nsfw: bool
    careful_bot: bool
    email_notification_types: list[str]
    has_pending_received_follow_request: bool
    has_unread_announcement: bool
    has_unread_antenna: bool
    has_unread_mentions: bool
    has_unread_messaging_message: bool
    has_unread_notification: bool
    has_unread_specified_notes: bool
    hide_online_status: bool
    inject_featured_note: bool
    integrations: dict[str, Any]
    is_deleted: bool
    is_explorable: bool
    muted_words: list[list[str]]
    muting_notification_types: list[str]
    no_crawle: bool
    receive_announcement_email: bool
    use_password_less_login: bool
    unread_announcements: list[IAnnouncement]
    two_factor_backup_codes_stock: Literal["full", "partial", "none"]


class IFollowRequest(TypedDict):
    id: str
    follower: ILiteUser
    followee: ILiteUser


def is_me_detailed(user: IUserDetailed | IMeDetailed, me_id: str) -> TypeGuard[IMeDetailed]:
    return user.get("avatar_id") is not None and user.get("id") == me_id



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.types.poll import ICreatePoll, IPoll, IPollChoice

if TYPE_CHECKING:
    from mipac.manager import ClientManager


class MiPoll:
    def __init__(self, poll: ICreatePoll) -> None:
        self.multiple = poll.get("multiple", False)
        self.choices = poll.get("choices")
        self.expired_after = poll.get("expired_after")
        self.expires_at = poll.get("expires_at")


class PollChoice(AbstractModel):
    def __init__(self, choice: IPollChoice, *, client: ClientManager):
        self.__choice: IPollChoice = choice
        self.__client: ClientManager = client

    @property
    def is_voted(self) -> bool:
        return self.__choice["is_voted"]

    @property
    def text(self) -> str:
        return self.__choice["text"]

    @property
    def votes(self) -> int:
        return self.__choice["votes"]


class Poll(AbstractModel):
    def __init__(self, poll: IPoll, *, client: ClientManager):
        self.__poll: IPoll = poll
        self.__client: ClientManager = client

    @property
    def multiple(self) -> bool:
        return self.__poll["multiple"]

    @property
    def expires_at(self) -> int:
        return self.__poll["expires_at"]

    @property
    def expired_after(self) -> int:
        return self.__poll["expired_after"]

    @property
    def choices(self) -> list[PollChoice]:
        return [PollChoice(i, client=self.__client) for i in self.__poll["choices"]]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion, NotSupportVersionText, ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.chat import ChatMessage
from mipac.types.chat import IChatMessage
from mipac.utils.util import check_multi_arg

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class BaseChatAction(AbstractAction):
    def __init__(
        self,
        session: HTTPClient,
        client: ClientManager,
        *,
        user_id: str | None = None,
        message_id: str | None = None,
    ):
        self.__session = session
        self.__client = client
        self.__user_id = user_id
        self.__message_id = message_id

    async def read(self, message_id: str | None = None) -> bool:
        """
        指定したIdのメッセージを既読にします

        Parameters
        ----------
        message_id : str
            Message id

        Returns
        -------
        bool
            Success or Failure.
        """
        if (
            self.__client._config.use_version >= 13
            and self.__client._config.features.chat is False
        ):
            raise NotSupportVersion(NotSupportVersionText)

        if check_multi_arg(message_id, self.__message_id) is False:
            raise ParameterError("message_idがありません")
        message_id = message_id or self.__message_id
        body = {"messageId": message_id}
        res: bool = await self.__session.request(
            Route("POST", "/api/messaging/messages/read"),
            json=body,
            auth=True,
            lower=True,
        )
        return res

    async def delete(self, message_id: str | None = None) -> bool:
        """
        指定したidのメッセージを削除します。

        Parameters
        ----------
        message_id : str
            Message id

        Returns
        -------
        bool
            Success or Failure.
        """
        if (
            self.__client._config.use_version >= 13
            and self.__client._config.features.chat is False
        ):
            raise NotSupportVersion(NotSupportVersionText)

        if check_multi_arg(message_id, self.__message_id) is False:
            raise ParameterError("message_idがありません")

        message_id = message_id or self.__message_id
        body = {"messageId": f"{message_id}"}
        res: bool = await self.__session.request(
            Route("POST", "/api/messaging/messages/delete"),
            json=body,
            auth=True,
        )
        return bool(res)


class ChatAction(BaseChatAction):
    def __init__(
        self,
        session: HTTPClient,
        client: ClientManager,
        *,
        user_id: str | None = None,
        message_id: str | None = None,
    ):
        super().__init__(session, client, user_id=user_id, message_id=message_id)
        self.__session = session
        self.__client = client
        self.__user_id = user_id
        self.__message_id = message_id

    async def get_history(self, limit: int = 100, group: bool = True):
        """
        Get the chat history.

        Parameters
        ----------
        limit : int, default=100, max=100
            Number of items to retrieve, up to 100
        group : bool, default=True
            Whether to include group chat or not

        Returns
        -------
        list[ChatMessage]
            List of chat history
        """
        if (
            self.__client._config.use_version >= 13
            and self.__client._config.features.chat is False
        ):
            raise NotSupportVersion(NotSupportVersionText)

        if limit > 100:
            raise ParameterError("limit must be greater than 100")

        args = {"limit": limit, "group": group}
        data: list[IChatMessage] = await self.__session.request(
            Route("POST", "/api/messaging/history"), json=args, auth=True
        )
        return [ChatMessage(d, client=self.__client) for d in data]

    async def send(
        self,
        text: str | None = None,
        *,
        file_id: str | None = None,
        user_id: str | None = None,
        group_id: str | None = None,
    ) -> ChatMessage:
        """
        Send chat.

        Parameters
        ----------
        text : str | None, default=None
            Chat content
        file_id : str | None, default=None
            添付するファイルのID
        user_id : str | None, default=None
            送信するユーザーのID
        group_id : str | None, default=None
            Destination group id
        """
        if (
            self.__client._config.use_version >= 13
            and self.__client._config.features.chat is False
        ):
            raise NotSupportVersion(NotSupportVersionText)
        user_id = user_id or self.__user_id
        data = {
            "userId": user_id,
            "groupId": group_id,
            "text": text,
            "fileId": file_id,
        }
        res = await self.__session.request(
            Route("POST", "/api/messaging/messages/create"),
            json=data,
            auth=True,
            lower=True,
        )
        return ChatMessage(res, client=self.__client)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.drive import (
    ClientFileActions,
    ClientFolderActions,
    DriveActions,
    FileActions,
    FolderActions,
)
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ["FolderManager", "FileManager", "DriveManager"]


class ClientFileManager(AbstractManager):
    def __init__(
        self,
        file_id: str | None = None,
        folder_id: str | None = None,
        url: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__file_id: str | None = file_id
        self.__folder_id: str | None = folder_id
        self.__url: str | None = url

    @property
    def action(self) -> ClientFileActions:
        return ClientFileActions(
            file_id=self.__file_id,
            folder_id=self.__folder_id,
            url=self.__url,
            client=self.__client,
            session=self.__session,
        )


class FileManager(AbstractManager):
    def __init__(
        self,
        file_id: str | None = None,
        folder_id: str | None = None,
        url: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__file_id = file_id
        self.__folder_id: str | None = folder_id
        self.__url: str | None = url

    @property
    def action(self) -> FileActions:
        """
        ファイルの操作を行うインスタンスを返します

        Return
        ------
        FileActions
            ファイルに対するアクション
        """
        return FileActions(
            file_id=self.__file_id,
            folder_id=self.__folder_id,
            url=self.__url,
            client=self.__client,
            session=self.__session,
        )


class ClientFolderManager(AbstractManager):
    def __init__(
        self, folder_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        self.__folder_id = folder_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.file: FileManager = FileManager(folder_id=folder_id, session=session, client=client)

    @property
    def action(self) -> ClientFolderActions:
        """
        フォルダーの操作を行うインスタンスを返します

        Returns
        -------
        FolderActions
            フォルダーに対するアクション
        """
        return ClientFolderActions(
            folder_id=self.__folder_id,
            session=self.__session,
            client=self.__client,
        )


class FolderManager(AbstractManager):
    def __init__(
        self, folder_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        self.__folder_id = folder_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.file: FileManager = FileManager(folder_id=folder_id, session=session, client=client)

    @property
    def action(self) -> FolderActions:
        """
        フォルダーの操作を行うインスタンスを返します

        Returns
        -------
        FolderActions
            フォルダーに対するアクション
        """
        return FolderActions(
            folder_id=self.__folder_id,
            session=self.__session,
            client=self.__client,
        )

    def _get_file_instance(self, file_id: str) -> FileManager:
        return FileManager(
            file_id=file_id,
            folder_id=self.__folder_id,
            session=self.__session,
            client=self.__client,
        )


class DriveManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.folder: FolderManager = FolderManager(session=session, client=client)
        self.file: FileManager = FileManager(session=session, client=client)

    @property
    def action(self) -> DriveActions:
        """
        ドライブの操作を行うインスタンスを返します

        Returns
        -------
        DriveActions
            ドライブに対するアクション
        """
        return DriveActions(client=self.__client, session=self.__session)

    def _get_folder_instance(self, folder_id: str) -> FolderManager:
        return FolderManager(session=self.__session, client=self.__client, folder_id=folder_id)

    def _get_client_folder_instance(self, folder_id: str) -> ClientFolderManager:
        return ClientFolderManager(
            folder_id=folder_id, session=self.__session, client=self.__client
        )

    def _get_client_file_instance(
        self, *, file_id: str, url: str, folder_id: str | None = None
    ) -> ClientFileManager:
        return ClientFileManager(
            file_id=file_id,
            folder_id=folder_id,
            url=url,
            session=self.__session,
            client=self.__client,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import APIError, ParameterError
from mipac.file import MiFile
from mipac.http import HTTPClient, Route
from mipac.models.clip import Clip
from mipac.models.drive import File
from mipac.models.note import Note, NoteReaction, NoteState, NoteTranslateResult
from mipac.models.poll import MiPoll, Poll
from mipac.types.clip import IClip
from mipac.types.note import ICreatedNote, INote, INoteState, INoteTranslateResult, INoteVisibility
from mipac.utils.cache import cache
from mipac.utils.format import remove_dict_empty
from mipac.utils.pagination import Pagination
from mipac.utils.util import check_multi_arg

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ["NoteActions"]


def create_note_body(
    content: str | None = None,
    visibility: INoteVisibility = "public",
    visible_user_ids: list[str] | None = None,
    cw: str | None = None,
    local_only: bool = False,
    extract_mentions: bool = True,
    extract_hashtags: bool = True,
    extract_emojis: bool = True,
    reply_id: str | None = None,
    renote_id: str | None = None,
    channel_id: str | None = None,
    files: list[MiFile | File | str] | None = None,
    poll: MiPoll | None = None,
):
    content = content or None
    body = {
        "visibility": visibility,
        "visibleUserIds": visible_user_ids,
        "text": content,
        "cw": cw,
        "localOnly": local_only,
        "noExtractMentions": not extract_mentions,
        "noExtractHashtags": not extract_hashtags,
        "noExtractEmojis": not extract_emojis,
        "replyId": reply_id,
        "renoteId": renote_id,
        "channelId": channel_id,
    }
    if not check_multi_arg(content, files, renote_id, poll):
        raise ParameterError("ノートの送信にはcontent, file_ids, renote_id またはpollのいずれか1つが無くてはいけません")

    if poll and type(Poll):
        poll_data = remove_dict_empty(
            {
                "choices": poll.choices,
                "multiple": poll.multiple,
                "expiresAt": poll.expires_at,
                "expiredAfter": poll.expired_after,
            }
        )
        body["poll"] = poll_data
    if files:
        file_ids = []
        for file in files:
            if isinstance(file, MiFile):
                file_ids.append(file.file_id)
            elif isinstance(file, File):
                file_ids.append(file.id)
            elif isinstance(file, str):
                file_ids.append(file)
            else:
                raise ParameterError("files must be MiFile or str or File")
        body["fileIds"] = file_ids

    return remove_dict_empty(body)


class ClientNoteActions(AbstractAction):
    def __init__(
        self,
        note_id: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        self._note_id: str | None = note_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def un_renote(self, note_id: str | None = None) -> bool:
        """
        Releases the note renote for the specified Id

        Parameters
        ----------
        note_id : str | None, optional
            Target note Id., by default None

        Returns
        -------
        bool
            Whether the release was successful
        """
        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        body = {"noteId": note_id}
        res: bool = await self._session.request(
            Route("POST", "/api/notes/unrenote"), auth=True, json=body
        )
        return res

    async def get_children(
        self,
        limit: int = 100,
        since_id: str | None = None,
        untilId: str | None = None,
        note_id: str | None = None,
        get_all: bool = True,
    ) -> AsyncGenerator[Note, None]:
        if limit > 100:
            raise ParameterError("limit は100以下である必要があります")

        if get_all:
            limit = 100

        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {
            "noteId": note_id,
            "limit": limit,
            "sinceId": since_id,
            "untilId": untilId,
        }

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/notes/children"), json=data
        )

        while True:
            res_notes = await pagination.next()
            for note in res_notes:
                yield Note(note, self._client)

            if get_all is False or pagination.is_final:
                break

    async def get_state(self, note_id: str | None = None) -> NoteState:
        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {"noteId": note_id}
        res: INoteState = await self._session.request(
            Route("POST", "/api/notes/state"), auth=True, json=data
        )
        return NoteState(res)

    async def add_clips(self, clip_id: str, note_id: str | None = None) -> bool:
        """
        クリップに追加します

        Parameters
        ----------
        note_id : str | None, default=None
                追加するノートのID
        clip_id : str
            クリップのID
        note_id : str | None, default=None
            追加したいノートのID

        Returns
        -------
        bool
            成功したか否か
        """

        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {"noteId": note_id, "clipId": clip_id}
        return bool(
            await self._session.request(Route("POST", "/api/clips/add-note"), json=data, auth=True)
        )

    async def get_clips(self, note_id: str | None = None) -> list[Clip]:
        """
        クリップを取得します

        Parameters
        ----------
        note_id : str | None, default=None
            取得したいノートのID

        Returns
        -------
        list[Clip]
            クリップのリスト
        """

        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {"noteId": note_id}
        res: list[IClip] = await self._session.request(
            Route("POST", "/api/notes/clips"), json=data, auth=True
        )
        return [Clip(clip, client=self._client) for clip in res]

    async def delete(self, note_id: str | None = None) -> bool:
        """
        Delete a note

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        bool
            success or not
        """

        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {"noteId": note_id}
        res = await self._session.request(Route("POST", "/api/notes/delete"), json=data, auth=True)
        return bool(res)

    async def create_renote(self, note_id: str | None = None) -> Note:
        """
        Renote a note

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        Note
            Renoted note
        """

        note_id = self._note_id or note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        body = create_note_body(renote_id=note_id)
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )
        return Note(res["created_note"], client=self._client)

    async def get_reaction(self, reaction: str, note_id: str | None = None) -> list[NoteReaction]:
        note_id = note_id or self._note_id
        return await self._client.note.reaction.action.get_reaction(
            reaction
        )  # TODO: note.reactionのインタンスを新規作成出来るように

    async def reply(
        self,
        content: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        reply_id: str | None = None,
    ) -> Note:
        reply_id = reply_id or self._note_id

        if reply_id is None:
            raise ParameterError("reply_id is required")

        body = create_note_body(
            content=content,
            cw=cw,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            extract_emojis=extract_emojis,
            extract_hashtags=extract_hashtags,
            extract_mentions=extract_mentions,
            poll=poll,
            local_only=local_only,
            reply_id=reply_id,
            files=files,
        )
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            lower=True,
            auth=True,
        )
        return Note(res["created_note"], client=self._client)

    async def create_quote(
        self,
        content: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        note_id: str | None = None,
    ) -> Note:
        """
        Create a note quote.

        Parameters
        ----------
        content: str | None, default=None
            text
        visibility: INoteVisibility, default='public'
            Disclosure range
        visible_user_ids: list[str] | None, default=None
            List of users to be published
        cw: str | None, default=None
            Text to be displayed when warning is given
        local_only: bool, default=False
            Whether to show only locally or not
        extract_mentions: bool, default=True
            Whether to expand the mention
        extract_hashtags: bool, default=True
            Whether to expand the hashtag
        extract_emojis: bool, default=True
            Whether to expand the emojis
        files: list[MiFile | File | str] | None, default=None
            The ID list of files to be attached
        poll: MiPoll | None, default=None
            Questionnaire to be created
        note_id: str | None, default=None
            Note IDs to target for renote and citations
        """

        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        body = create_note_body(
            content=content,
            cw=cw,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            extract_emojis=extract_emojis,
            extract_hashtags=extract_hashtags,
            extract_mentions=extract_mentions,
            poll=poll,
            local_only=local_only,
            renote_id=note_id,
            files=files,
        )
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )

        return Note(res["created_note"], client=self._client)

    @cache(group="translate_note")
    async def translate(
        self,
        note_id: str | None = None,
        target_lang: str = "en-US",
    ) -> NoteTranslateResult:
        """
        Translate a note

        Parameters
        ----------
        note_id : str | None, default=None
            Note ID to target for translation
        target_lang : str, default='en'
            Target language

        Returns
        -------
        NoteTranslateResult
            Translated result
        """
        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {"noteId": note_id, "targetLang": target_lang}
        res: INoteTranslateResult = await self._session.request(
            Route("POST", "/api/notes/translate"), json=data, auth=True
        )
        if isinstance(res, dict):
            return NoteTranslateResult(res)
        APIError(f"Translate Error: {res}", res if isinstance(res, int) else 204).raise_error()

    async def get_replies(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        note_id: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[Note, None]:
        """
        ノートに対する返信を取得します

        Parameters
        ---------
        since_id : str | None, default=None
            指定すると、その投稿を投稿を起点としてより新しい投稿を取得します
        until_id : str | None, default=None
            指定すると、その投稿を投稿を起点としてより古い投稿を取得します
        limit : int, default=10
            取得する上限
        note_id: str | None, default=None
            返信を取得したいノートのID

        Returns
        -------
        AsyncGenerator[Note, None]
            返信
        """

        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        note_id = note_id or self._note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        body = {"noteId": note_id, "sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[INote](self._session, Route("POST", "/api/notes"), json=body)

        while True:
            res_notes = await pagination.next()
            for res_note in res_notes:
                yield Note(res_note, client=self._client)

            if get_all is False or pagination.is_final:
                break


class NoteActions(ClientNoteActions):
    def __init__(
        self,
        note_id: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(note_id=note_id, session=session, client=client)

    async def send(
        self,
        content: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        reply_id: str | None = None,
        renote_id: str | None = None,
        channel_id: str | None = None,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
    ) -> Note:
        """
        ノートを投稿します。

        Parameters
        ----------
        content : str | None, default=None
            投稿する内容
        visibility : INoteVisibility, optional
            公開範囲, by default "public"
            Enum: "public" "home" "followers" "specified"
        visible_user_ids : list[str] | None, optional
            公開するユーザー, by default None
        cw : str | None, optional
            閲覧注意の文字, by default None
        local_only : bool, optional
            ローカルにのみ表示するか, by default False
        extract_mentions : bool, optional
            メンションを展開するか, by default False
        extract_hashtags : bool, optional
            ハッシュタグを展開するか, by default False
        extract_emojis : bool, optional
            絵文字を展開するか, by default False
        reply_id : str | None, optional
            リプライ先のid, by default None
        renote_id : str | None, optional
            リノート先のid, by default None
        channel_id : str | None, optional
            チャンネルid, by default None
        files : list[MiFile | File | str], optional
            添付するファイルのリスト, by default None
        poll : MiPoll | None, optional
            アンケート, by default None

        Returns
        -------
        Note
            投稿したノート

        Raises
        ------
        ContentRequired
            [description]
        """
        body = create_note_body(
            content=content,
            cw=cw,
            channel_id=channel_id,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            extract_emojis=extract_emojis,
            extract_hashtags=extract_hashtags,
            extract_mentions=extract_mentions,
            files=files,
            poll=poll,
            local_only=local_only,
            renote_id=renote_id,
            reply_id=reply_id,
        )
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )
        return Note(res["created_note"], client=self._client)

    @cache(group="get_note")
    async def get(self, note_id: str) -> Note:
        """
        ノートを取得します

        Parameters
        ----------
        note_id : str
            ノートのID

        Returns
        -------
        Note
            取得したノートID
        """
        res = await self._session.request(
            Route("POST", "/api/notes/show"),
            json={"noteId": note_id},
            auth=True,
            lower=True,
        )
        return Note(res, client=self._client)

    @cache(group="get_note", override=True)
    async def fetch(self, note_id: str) -> Note:
        res = await self._session.request(
            Route("POST", "/api/notes/show"),
            json={"noteId": note_id},
            auth=True,
            lower=True,
        )
        return Note(res, client=self._client)

    async def gets(
        self,
        local: bool = False,
        reply: bool = False,
        renote: bool = False,
        with_files: bool = False,
        poll: bool = False,
        limit: int = 100,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[Note, None]:
        if limit > 100:
            raise ParameterError("limit は100以下である必要があります")

        if get_all:
            limit = 100

        body = remove_dict_empty(
            {
                "local": local,
                "reply": reply,
                "renote": renote,
                "withFiles": with_files,
                "poll": poll,
                "limit": limit,
                "sinceId": since_id,
                "untilId": until_id,
            }
        )

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/notes"), json=body, limit=limit
        )

        while True:
            res_notes = await pagination.next()
            for note in res_notes:
                yield Note(note, client=self._client)
            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from mipac.abstract.model import AbstractModel
from mipac.types.instance import IInstanceLite


class LiteInstance(AbstractModel):
    def __init__(self, instance: IInstanceLite) -> None:
        self.__instance: IInstanceLite = instance

    @property
    def name(self) -> str:
        return self.__instance["name"]

    def software_name(self) -> str:
        return self.__instance["software_name"]

    def software_version(self) -> str:
        return self.__instance["software_version"]

    def icon_url(self) -> str:
        return self.__instance["icon_url"]

    def favicon_url(self) -> str:
        return self.__instance["favicon_url"]

    def theme_color(self) -> str:
        return self.__instance["theme_color"]



class MiFile:
    __slots__ = (
        "path",
        "file_id",
        "name",
        "folder_id",
        "comment",
        "is_sensitive",
        "force",
    )

    def __init__(
        self,
        path: str | None = None,
        file_id: str | None = None,
        name: str | None = None,
        folder_id: str | None = None,
        comment: str | None = None,
        is_sensitive: bool = False,
        force: bool = False,
    ):
        """
        Parameters
        ----------
        path : str | None, default=None
            path to a local file
        file_id : str | None, default=None
            ID of the file that exists on the drive
        name str | None, default=None
            file name
        folder_id : str | None, default=None
            Folder ID
        comment : str | None, default=None
            Comments on files
        is_sensitive : str | None, default=None
            Whether this item is sensitive
        force : bool, default=False
            Whether to force overwriting even if it already exists on the drive
        """
        self.path = path
        self.file_id = file_id
        self.name = name
        self.folder_id = folder_id
        self.comment = comment
        self.is_sensitive = is_sensitive
        self.force = force



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.drive import File
from mipac.models.lite.user import LiteUser

if TYPE_CHECKING:
    from mipac.actions.chat import BaseChatAction
    from mipac.manager.client import ClientManager
    from mipac.types.chat import IChatGroup, IChatMessage

__all__ = ["ChatGroup", "ChatMessage"]


class ChatGroup(AbstractModel):
    def __init__(self, group: IChatGroup, *, client: ClientManager):
        self.__group: IChatGroup = group
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """グループのID"""
        return self.__group["id"]

    @property
    def created_at(self) -> str:
        """グループの作成日時"""
        return self.__group["created_at"]

    @property
    def name(self) -> str:
        """グループ名"""
        return self.__group["name"]

    @property
    def owner_id(self) -> str:
        """グループのオーナーのID"""
        return self.__group["owner_id"]

    @property
    def user_ids(self) -> list[str]:
        return self.__group["user_ids"]

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, ChatGroup) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class ChatMessage(AbstractModel):
    """
    チャットオブジェクト
    """

    def __init__(self, chat: IChatMessage, *, client: ClientManager):
        self.__chat: IChatMessage = chat
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """The message ID."""
        return self.__chat["id"]

    @property
    def created_at(self) -> str:
        """Returns the date and time the message was created (UTC)"""
        return self.__chat["created_at"]

    @property
    def file(self) -> File | None:
        return File(self.__chat["file"], client=self.__client) if self.__chat["file"] else None

    @property
    def text(self) -> str | None:
        """text of the message"""
        return self.__chat["text"]

    @property
    def user_id(self) -> str:
        return self.__chat["user_id"]

    @property
    def user(self) -> LiteUser:
        return LiteUser(self.__chat["user"], client=self.__client)

    @property
    def recipient_id(self) -> str | None:
        """The id of the bot self"""
        return self.__chat["recipient_id"]

    @property
    def recipient(self) -> LiteUser | None:
        """The user of the bot self"""

        return (
            LiteUser(user=self.__chat["recipient"], client=self.__client)
            if self.__chat["recipient"]
            else None
        )

    @property
    def group_id(self) -> str | None:
        return self.__chat["group_id"]

    @property
    def file_id(self) -> str | None:
        return self.__chat["file_id"]

    @property
    def is_read(self) -> bool:
        return bool(self.__chat["is_read"])

    @property
    def reads(self) -> list[str]:
        return self.__chat["reads"]

    @property
    def group(self) -> ChatGroup | None:
        return (
            ChatGroup(self.__chat["group"], client=self.__client) if self.__chat["group"] else None
        )

    @property
    def api(self) -> BaseChatAction:
        return self.__client.chat.custom_base_chat_action(user_id=self.user.id, message_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, ChatMessage) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



"""
MiPAを使用する上でちょっとした際に便利なツール一覧
"""
from __future__ import annotations

import asyncio
import functools
import json
import re
import uuid
import warnings
from datetime import datetime, timedelta
from typing import Any, Mapping, Optional
from urllib.parse import urlencode

import aiohttp

from mipac.utils.util import deprecated as new_deprecated

try:
    import orjson  # type: ignore
except ModuleNotFoundError:
    HAS_ORJSON = False
else:
    HAS_ORJSON = True

__all__ = (
    "deprecated",
    "MiTime",
    "get_cache_key",
    "key_builder",
    "check_multi_arg",
    "remove_list_empty",
    "remove_dict_empty",
    "upper_to_lower",
    "str_lower",
    "bool_to_string",
    "_from_json",
    "str_to_datetime",
    "convert_dict_keys_to_camel",
    "snake_to_camel",
)


if HAS_ORJSON:
    _from_json = orjson.loads  # type: ignore
else:
    _from_json = json.loads

DEFAULT_CACHE: dict[str, list[str]] = {}
DEFAULT_CACHE_VALUE: dict[str, Any] = {}


@new_deprecated
def deprecated(func):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.
    """

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.simplefilter("always", DeprecationWarning)  # turn off filter
        warnings.warn(
            "Call to deprecated function {}.".format(func.__name__),
            category=DeprecationWarning,
            stacklevel=2,
        )
        warnings.simplefilter("default", DeprecationWarning)  # reset filter
        return func(*args, **kwargs)

    return new_func


@new_deprecated
def snake_to_camel(snake_str: str, replace_list: dict[str, str]) -> str:
    components: list[str] = snake_str.split("_")
    for i in range(len(components)):
        if components[i] in replace_list:
            components[i] = replace_list[components[i]]
    return components[0] + "".join(x.title() for x in components[1:])


@new_deprecated
def convert_dict_keys_to_camel(
    data: Mapping[Any, Any], replace_list: dict[str, str] | None = None
) -> Mapping[Any, Any]:
    if replace_list is None:
        replace_list = {}
    new_dict = {}
    for key, value in data.items():
        new_key = snake_to_camel(key, replace_list)
        new_dict[new_key] = value
    return new_dict


@new_deprecated
def str_to_datetime(data: str, format: str = "%Y-%m-%dT%H:%M:%S.%fZ") -> datetime:
    """
    Parameters
    ----------
    data : str
        datetimeに変更したい文字列
    format : str
        dataのフォーマット

    Returns
    -------
    datetime
        変換後のデータ
    """
    return datetime.strptime(data, format)


@new_deprecated
class MiTime:
    def __init__(self, start: timedelta, end: datetime):
        self.start = start
        self.end = end


@new_deprecated
class AuthClient:
    """
    Tokenの取得を手助けするクラス
    """

    def __init__(
        self,
        instance_uri: str,
        name: str,
        description: str,
        permissions: Optional[list[str]] = None,
        *,
        icon: str | None = None,
        use_miauth: bool = False,
    ):
        """
        Parameters
        ----------
        instance_uri : str
            アプリケーションを作成したいインスタンスのURL
        name : str
            アプリケーションの名前
        description : str
            アプリケーションの説明
        permissions : Optional[list[str]], default=None
            アプリケーションが要求する権限
        icon: str | None, default=None
            アプリケーションのアイコン画像URL
        use_miauth: bool, default=False
            MiAuthを使用するか
        """
        if permissions is None:
            permissions = ["read:account"]
        self.__client_session = aiohttp.ClientSession()
        self.__instance_uri: str = instance_uri
        self.__name: str = name
        self.__description: str = description
        self.__permissions: list[str] = permissions
        self.__icon: str | None = icon
        self.__use_miauth: bool = use_miauth
        self.__session_token: uuid.UUID
        self.__secret: str

    async def get_auth_url(self) -> str:
        """
        認証に使用するURLを取得します

        Returns
        -------
        str
            認証に使用するURL
        """
        field = remove_dict_empty(
            {"name": self.__name, "description": self.__description, "icon": self.__icon}
        )
        if self.__use_miauth:
            field["permissions"] = self.__permissions
            query = urlencode(field)
            self.__session_token = uuid.uuid4()
            return f"{self.__instance_uri}/miauth/{self.__session_token}?{query}"
        else:
            field["permission"] = self.__permissions
            async with self.__client_session.post(
                f"{self.__instance_uri}/api/app/create", json=field
            ) as res:
                data = await res.json()
                self.__secret = data["secret"]
            async with self.__client_session.post(
                f"{self.__instance_uri}/api/auth/session/generate",
                json={"appSecret": self.__secret},
            ) as res:
                data = await res.json()
                self.__session_token = data["token"]
                return data["url"]

    async def wait_miauth(self) -> str:
        url = f"{self.__instance_uri}/api/miauth/{self.__session_token}/check"
        while True:
            async with self.__client_session.post(url) as res:
                data = await res.json()
                if data.get("ok") is True:
                    return data
            await asyncio.sleep(1)

    async def wait_oldauth(self) -> None:
        while True:
            async with self.__client_session.post(
                f"{self.__instance_uri}/api/auth/session/userkey",
                json={"appSecret": self.__secret, "token": self.__session_token},
            ) as res:
                data = await res.json()
                if data.get("error", {}).get("code") != "PENDING_SESSION":
                    break
            await asyncio.sleep(1)

    async def check_auth(self) -> str:
        """
        認証が完了するまで待機し、完了した場合はTokenを返します

        Returns
        -------
        str
            Token
        """
        if self.__use_miauth:
            data = await self.wait_miauth()
        else:
            data = await self.wait_oldauth()
        await self.__client_session.close()
        return data["token"] if self.__use_miauth else data["accessToken"]


@new_deprecated
def set_cache(group: str, key: str, value: Any):
    if len(DEFAULT_CACHE.get(group, [])) > 50:
        del DEFAULT_CACHE[group][-1]
        del DEFAULT_CACHE_VALUE[key]

    if DEFAULT_CACHE.get(group) is None:
        DEFAULT_CACHE[group] = []
    DEFAULT_CACHE[group].append(key)
    DEFAULT_CACHE_VALUE[key] = value


@new_deprecated
def cache(group: str = "default", override: bool = False):
    def decorator(func):
        async def wrapper(self, *args, **kwargs):
            ordered_kwargs = sorted(kwargs.items())
            key = ".{0}" + str(args) + str(ordered_kwargs)
            hit_item = DEFAULT_CACHE_VALUE.get(key)
            if hit_item and override is False and kwargs.get("cache_override") is None:
                return hit_item
            res = await func(self, *args, **kwargs)
            set_cache(group, key, res)
            return res

        return wrapper

    return decorator


@new_deprecated
def get_cache_key(func):
    async def decorator(self, *args, **kwargs):
        ordered_kwargs = sorted(kwargs.items())
        key = (func.__module__ or "") + ".{0}" + f"{self}" + str(args) + str(ordered_kwargs)
        return await func(self, *args, **kwargs, cache_key=key)

    return decorator


@new_deprecated
def key_builder(func, cls, *args, **kwargs):
    ordered_kwargs = sorted(kwargs.items())
    key = (
        (func.__module__ or "") + f".{func.__name__}" + f"{cls}" + str(args) + str(ordered_kwargs)
    )
    return key


@new_deprecated
def check_multi_arg(*args: Any) -> bool:
    """
    複数の値を受け取り値が存在するかをboolで返します

    Parameters
    ----------
    args : list
        確認したい変数のリスト

    Returns
    -------
    bool
        存在する場合はTrue, 存在しない場合はFalse
    """
    return bool([i for i in args if i])


@new_deprecated
def remove_list_empty(data: list[Any]) -> list[Any]:
    """
    Parameters
    ----------
    data: dict
        空のkeyを削除したいdict

    Returns
    -------
    dict[str, Any]
        空のkeyがなくなったdict
    """
    return [k for k in data if k]


@new_deprecated
def remove_dict_empty(data: dict[str, Any]) -> dict[str, Any]:
    """
    Parameters
    ----------
    data: dict
        空のkeyを削除したいdict

    Returns
    -------
    _data: dict
        空のkeyがなくなったdict
    """
    _data = {}
    _data = {k: v for k, v in data.items() if v is not None}
    return _data


@new_deprecated
def upper_to_lower(
    data: dict[str, Any],
    field: Optional[dict[str, Any]] = None,
    nest: bool = True,
    replace_list: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
    """
    Parameters
    ----------
    data: dict
        小文字にしたいkeyがあるdict
    field: dict, default=None
        謎
    nest: bool, default=True
        ネストされたdictのkeyも小文字にするか否か
    replace_list: dict, default=None
        dictのkey名を特定の物に置き換える

    Returns
    -------
    field : dict
        小文字になった, key名が変更されたdict
    """
    if data is None:
        return {}
    if replace_list is None:
        replace_list = {}

    if field is None:
        field = {}
    for attr in data:
        pattern = re.compile("[A-Z]")
        large = [i.group().lower() for i in pattern.finditer(attr)]
        result = [None] * (len(large + pattern.split(attr)))
        result[::2] = pattern.split(attr)
        result[1::2] = ["_" + i.lower() for i in large]
        default_key = "".join(result)
        if replace_list.get(attr):
            default_key = default_key.replace(attr, replace_list.get(attr))
        field[default_key] = data[attr]
        if isinstance(field[default_key], dict) and nest:
            field[default_key] = upper_to_lower(field[default_key])
        elif isinstance(field[default_key], list) and nest:
            field[default_key] = [
                upper_to_lower(i) if isinstance(i, dict) else i for i in field[default_key]
            ]
    return field


@new_deprecated
def str_lower(text: str):
    pattern = re.compile("[A-Z]")
    large = [i.group().lower() for i in pattern.finditer(text)]
    result = [None] * (len(large + pattern.split(text)))
    result[::2] = pattern.split(text)
    result[1::2] = ["_" + i.lower() for i in large]
    return "".join(result)


@new_deprecated
def bool_to_string(boolean: bool) -> str:
    """
    boolを小文字にして文字列として返します

    Parameters
    ----------
    boolean : bool
        変更したいbool値
    Returns
    -------
    true or false: str
        小文字になったbool文字列
    """
    return "true" if boolean else "false"



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotExistRequiredData, ParameterError
from mipac.http import Route
from mipac.models.emoji import CustomEmoji
from mipac.types.emoji import ICustomEmoji
from mipac.utils.pagination import Pagination
from mipac.utils.util import check_multi_arg

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager


class AdminEmojiActions(AbstractAction):
    def __init__(self, emoji_id: None | str = None, *, session: HTTPClient, client: ClientManager):
        self.__emoji_id = emoji_id
        self.__session = session
        self.__client = client

    async def add(
        self,
        file_id: str | None = None,
        *,
        name: str | None = None,
        url: str | None = None,
        category: str | None = None,
        aliases: list[str] | None = None,
    ) -> bool:
        """絵文字を追加します

        Parameters
        ----------
        file_id : str | None, optional
            追加する絵文字のファイルId, by default None
        name : str | None, optional
            絵文字名, by default None
        url : str | None, optional
            絵文字があるUrl, by default None
        category : str | None, optional
            絵文字のカテゴリー, by default None
        aliases : list[str] | None, optional
            絵文字のエイリアス, by default None

        Returns
        -------
        bool
            成功したかどうか

        Raises
        ------
        NotExistRequiredData
            必要なデータが不足している
        """

        if self.__client._config.use_version >= 12:
            data = {"fileId": file_id}
        else:
            data = {
                "name": name,
                "url": url,
                "category": category,
                "aliases": aliases,
            }

        if not check_multi_arg(file_id, url):
            raise NotExistRequiredData("required a file_id or url")
        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/emoji/add"),
                json=data,
                lower=True,
                auth=True,
            )
        )

    async def gets(
        self,
        query: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        get_all: bool = True,
    ) -> AsyncGenerator[CustomEmoji, None]:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        body = {
            "query": query,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }

        pagination = Pagination[ICustomEmoji](
            self.__session, Route("POST", "/api/admin/emoji/list"), json=body
        )

        while True:
            res_custom_emojis = await pagination.next()
            for res_custom_emoji in res_custom_emojis:
                yield CustomEmoji(res_custom_emoji, client=self.__client)

            if get_all is False or pagination.is_final:
                break

    async def gets_remote(
        self,
        query: str | None = None,
        host: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        get_all: bool = True,
    ) -> AsyncGenerator[CustomEmoji, None]:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        body = {
            "query": query,
            "host": host,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }

        pagination = Pagination[ICustomEmoji](
            self.__session, Route("POST", "/api/admin/emoji/list-remote"), json=body
        )

        while True:
            res_custom_emojis = await pagination.next()
            for res_custom_emoji in res_custom_emojis:
                yield CustomEmoji(res_custom_emoji, client=self.__client)

            if get_all is False or pagination.is_final:
                break

    async def set_license_bulk(self, ids: list[str], license: str | None = None) -> bool:
        body = {"ids": ids, "license": license}
        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/set-license-bulk"),
            auth=True,
            json=body,
            remove_none=False,  # remove_noneをFalseにしないとlisenceが消せなくなる
        )
        return res

    async def remove(self, emoji_id: str | None = None) -> bool:
        """指定したIdの絵文字を削除します

        Parameters
        ----------
        emoji_id : str | None, optional
            削除する絵文字のId, by default None

        Returns
        -------
        bool
            成功したかどうか

        Raises
        ------
        NotExistRequiredData
            Idが不足している
        """

        emoji_id = emoji_id or self.__emoji_id

        if emoji_id is None:
            raise NotExistRequiredData("idが不足しています")

        endpoint = (
            "/api/admin/emoji/delete"
            if self.__client._config.use_version >= 12
            else "/api/admin/emoji/remove"
        )

        return bool(
            await self.__session.request(
                Route("POST", endpoint),
                auth=True,
                json={"id": emoji_id},
                lower=True,
            )
        )



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion, NotSupportVersionText, ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.ad import Ad
from mipac.types.ads import IAd
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminAdvertisingModelActions(AbstractAction):
    def __init__(self, ad_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self._ad_id: str | None = ad_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def update(
        self,
        url: str,
        place: Literal["square", "horizontal", "horizontal-big"],
        priority: Literal["high", "middle", "low"],
        ratio: int,
        image_url: str,
        id: str | None = None,
        starts_at: int = 0,
        expires_at: int = 0,
        memo: str | None = None,
    ) -> bool:
        ad_id = self._ad_id or id
        if ad_id is None:
            raise ParameterError("ad_idは必須です")
        data = {
            "id": ad_id,
            "url": url,
            "memo": memo or "",
            "place": place,
            "priority": priority,
            "ratio": ratio,
            "startsAt": starts_at,
            "expiresAt": expires_at,
            "imageUrl": image_url,
        }
        res: bool = await self._session.request(
            Route("POST", "/api/admin/ad/update"), json=data, auth=True, lower=True
        )
        return res

    async def delete(self, id: str | None = None) -> bool:
        if self._client._config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)

        ad_id = self._ad_id or id
        res: bool = await self._session.request(
            Route("POST", "/api/admin/ad/delete"), json={"id": ad_id}, auth=True, lower=True
        )
        return res


class AdminAdvertisingActions(AdminAdvertisingModelActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def create(
        self,
        url: str,
        place: Literal["square", "horizontal", "horizontal-big"],
        priority: Literal["high", "middle", "low"],
        ratio: int,
        image_url: str,
        starts_at: int = 0,
        expires_at: int = 0,
        memo: str | None = None,
    ) -> bool:
        if self._client._config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)

        data = {
            "url": url,
            "memo": memo or "",
            "place": place,
            "priority": priority,
            "ratio": ratio,
            "startsAt": starts_at,
            "expiresAt": expires_at,
            "imageUrl": image_url,
        }
        res: bool = await self._session.request(
            Route("POST", "/api/admin/ad/create"), json=data, auth=True, lower=True
        )
        return res

    async def get_list(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[Ad, None]:
        if self._client._config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)

        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")

        if get_all:
            limit = 100

        data = {"limit": limit, "sinceId": since_id, "untilId": until_id}

        pagination = Pagination[IAd](self._session, Route("POST", "/api/admin/ad/list"), json=data)

        while True:
            raw_ads = await pagination.next()
            for raw_ad in raw_ads:
                yield Ad(raw_ad, client=self._client)

            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.user import UserDetailed

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class AdminUserActions(AbstractAction):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id = user_id
        self.__session = session
        self.__client = client

    async def delete_account(self, user_id: str | None = None) -> bool:
        """
        Deletes the user with the specified user ID.

        Parameters
        ----------
        user_id : str | None, default=None
            ID of the user to be deleted
        Returns
        -------
        bool
            Success or failure
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        res = await self.__session.request(
            Route("POST", "/api/admin/accounts/delete"),
            json=data,
            auth=True,
            lower=True,
        )
        return bool(res)

    async def show_user(self, user_id: str | None = None) -> UserDetailed:
        """
        Shows the user with the specified user ID.

        Parameters
        ----------
        user_id : str | None, default=None
            ID of the user to be shown

        Returns
        -------
        UserDetailed
        """

        user_id = user_id or self.__user_id
        data = {"userId": user_id}
        res = await self.__session.request(
            Route("GET", "/api/admin/show-user"),
            json=data,
            auth=True,
            lower=True,
        )
        return UserDetailed(res, client=self.__client)

    async def suspend(self, user_id: str | None = None) -> bool:
        """
        Suspends the user with the specified user ID.

        Parameters
        ----------
        user_id : str | None, default=None
            ID of the user to be suspended

        Returns
        -------
        bool
            Success or failure
        """

        user_id = user_id or self.__user_id
        data = {"userId": user_id}
        res = await self.__session.request(
            Route("POST", "/api/admin/suspend-user"),
            json=data,
            auth=True,
            lower=True,
        )
        return bool(res)

    async def unsuspend(self, user_id: str | None = None) -> bool:
        """
        Unsuspends the user with the specified user ID.

        Parameters
        ----------
        user_id : str | None, default=None
            ID of the user to be unsuspended

        Returns
        -------
        bool
            Success or failure
        """

        user_id = user_id or self.__user_id
        data = {"userId": user_id}
        res: bool = await self.__session.request(
            Route("POST", "/api/admin/unsuspend-user"),
            json=data,
            auth=True,
            lower=True,
        )
        return bool(res)



from mipac.abstract.model import AbstractModel
from mipac.types.chart import IActiveUsersChart, IDriveChart, IDriveLocalChart, IDriveRemoteChart


class ActiveUsersChart(AbstractModel):
    __slots__ = ("__data",)

    def __init__(self, data: IActiveUsersChart):
        self.__data = data

    @property
    def read_write(self) -> list[int]:
        return self.__data["read_write"]

    @property
    def read(self) -> list[int]:
        return self.__data["read"]

    @property
    def write(self) -> list[int]:
        return self.__data["write"]

    @property
    def registered_within_week(self) -> list[int]:
        return self.__data["registered_within_week"]

    @property
    def registered_within_month(self) -> list[int]:
        return self.__data["registered_within_month"]

    @property
    def registered_within_year(self) -> list[int]:
        return self.__data["registered_within_year"]

    @property
    def registered_outside_week(self) -> list[int]:
        return self.__data["registered_outside_week"]

    @property
    def registered_outside_month(self) -> list[int]:
        return self.__data["registered_outside_month"]

    @property
    def registered_outside_year(self) -> list[int]:
        return self.__data["registered_outside_year"]


class DriveLocalChart(AbstractModel):
    __slots__ = ("__data",)

    def __init__(self, data: IDriveLocalChart):
        self.__data = data

    @property
    def total_count(self) -> list[int]:
        return self.__data["total_count"]

    @property
    def total_size(self) -> list[int]:
        return self.__data["total_size"]

    @property
    def inc_count(self) -> list[int]:
        return self.__data["inc_count"]

    @property
    def inc_size(self) -> list[int]:
        return self.__data["inc_size"]

    @property
    def dec_count(self) -> list[int]:
        return self.__data["dec_count"]

    @property
    def dec_size(self) -> list[int]:
        return self.__data["dec_size"]


class DriveRemoteChart(AbstractModel):
    __slots__ = ("__data",)

    def __init__(self, data: IDriveRemoteChart):
        self.__data: IDriveRemoteChart = data

    @property
    def total_count(self) -> list[int]:
        return self.__data["total_count"]

    @property
    def total_size(self) -> list[int]:
        return self.__data["total_size"]

    @property
    def inc_count(self) -> list[int]:
        return self.__data["inc_count"]

    @property
    def inc_size(self) -> list[int]:
        return self.__data["inc_size"]

    @property
    def dec_count(self) -> list[int]:
        return self.__data["dec_count"]

    @property
    def dec_size(self) -> list[int]:
        return self.__data["dec_size"]


class DriveChart(AbstractModel):
    __slots__ = ("__data",)

    def __init__(self, data: IDriveChart):
        self.__data: IDriveChart = data

    @property
    def local(self) -> DriveLocalChart:
        return DriveLocalChart(self.__data["local"])

    @property
    def remote(self) -> DriveRemoteChart:
        return DriveRemoteChart(self.__data["remote"])



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import Route

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager


class FavoriteActions(AbstractAction):
    def __init__(self, note_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__note_id: str | None = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def add(self, note_id: str | None = None) -> bool:
        note_id = note_id or self.__note_id
        data = {"noteId": note_id}
        return bool(
            await self.__session.request(
                Route("POST", "/api/notes/favorites/create"),
                json=data,
                auth=True,
            )
        )

    async def remove(self, note_id: str | None = None) -> bool:
        note_id = note_id or self.__note_id
        data = {"noteId": note_id}
        return bool(
            await self.__session.request(
                Route("POST", "/api/notes/favorites/delete"),
                json=data,
                auth=True,
            )
        )



from __future__ import annotations

import io
from http.client import HTTPException
from os import PathLike
from typing import TYPE_CHECKING, Any, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.drive import File, Folder
from mipac.types.drive import FolderPayload, IDriveFile
from mipac.utils.format import bool_to_string, remove_dict_empty
from mipac.utils.pagination import Pagination
from mipac.utils.util import deprecated

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager

__all__ = ("DriveActions", "FileActions", "FolderActions")


class ClientFileActions(AbstractAction):
    def __init__(
        self,
        file_id: str | None = None,
        folder_id: str | None = None,
        url: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ) -> None:
        self._session: HTTPClient = session
        self._client: ClientManager = client
        self._file_id = file_id
        self._folder_id = folder_id
        self._url = url

    async def remove(self, file_id: str | None = None) -> bool:
        """
        指定したIDのファイルを削除します

        Parameters
        ----------
        file_id : str | None, default=None
            削除するファイルのID

        Returns
        -------
        bool
            削除に成功したかどうか
        """

        file_id = file_id or self._file_id
        if file_id is None:
            raise ParameterError("file_id is required")

        return bool(
            await self._session.request(
                Route("POST", "/api/drive/files/delete"),
                json={"fileId": file_id},
                auth=True,
            )
        )

    async def save(
        self,
        fp: io.BufferedIOBase | PathLike[Any],
        file_id: str | None = None,
        url: str | None = None,
    ):
        file_id = file_id or self._file_id
        url = url or self._url
        if any([file_id, url]) is False:
            raise ParameterError("file_id is required")

        if url is None:
            result = await self._client.drive.file.action.show_file(file_id=file_id)
            url = result.url

        async with self._session.session.get(url) as resp:
            if resp.status == 200:
                content = await resp.read()
                with open(fp, "wb") as f:
                    return f.write(content)
            elif resp.status == 400:
                raise FileNotFoundError("File not found")
            else:
                raise HTTPException(resp, "Failed to get file")

    @deprecated
    async def remove_file(self, file_id: str | None = None) -> bool:
        """
        指定したIDのファイルを削除します

        Parameters
        ----------
        file_id : str | None, default=None
            削除するファイルのID

        Returns
        -------
        bool
            削除に成功したかどうか
        """

        file_id = file_id or self._file_id
        return bool(
            await self._session.request(
                Route("POST", "/api/drive/files/delete"),
                json={"fileId": file_id},
                auth=True,
            )
        )


class FileActions(ClientFileActions):
    def __init__(
        self,
        file_id: str | None = None,
        folder_id: str | None = None,
        url: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ) -> None:
        super().__init__(
            file_id=file_id, folder_id=folder_id, url=url, session=session, client=client
        )

    async def show_file(self, file_id: str | None = None, url: str | None = None) -> File:
        """
        ファイルの情報を取得します。

        Parameters
        ----------
        file_id : str | None, default=None
            ファイルのID
        url : str | None, default=None
            ファイルのURL

        Returns
        -------
        File
            ファイルの情報
        """

        data = remove_dict_empty({"fileId": file_id, "url": url})
        res: IDriveFile = await self._session.request(
            Route("POST", "/api/admin/drive/show-file"),
            json=data,
            auth=True,
            lower=True,
        )
        return File(res, client=self._client)

    async def get_files(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        folder_id: str | None = None,
        file_type: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[File, None]:
        """
        ファイルを取得します

        Parameters
        ----------
        limit : int, default=10
            取得する上限
        since_id : str | None, default=None
            指定すると、そのIDを起点としてより新しいファイルを取得します
        until_id : str | None, default=None
            指定すると、そのIDを起点としてより古いファイルを取得します
        folder_id : str | None, default=None
            指定すると、そのフォルダーを起点としてファイルを取得します
        file_type : str | None, default=None
            取得したいファイルの拡張子
        """
        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        folder_id = self._folder_id or folder_id

        body = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "folderId": folder_id,
            "Type": file_type,
        }

        pagination = Pagination[IDriveFile](
            self._session, Route("POST", "/api/drive/files"), json=body
        )

        while True:
            res_drive_files = await pagination.next()

            for res_drive_file in res_drive_files:
                yield File(res_drive_file, client=self._client)

            if get_all is False or pagination.is_final:
                break

    async def upload_file(
        self,
        file: str,
        file_name: str | None = None,
        folder_id: str | None = None,
        comment: str | None = None,
        is_sensitive: bool = False,
        force: bool = False,
    ) -> File:
        """
        ファイルをアップロードします

        Parameters
        ----------
        file : str
            アップロードするファイル
        file_name : str | None, default=None
            アップロードするファイルの名前
        folder_id : str | None, default=None
            アップロードするフォルダーのID
        comment : str | None, default=None
            アップロードするファイルのコメント
        is_sensitive : bool, default=False
            アップロードするファイルがNSFWかどうか
        force : bool, default=False
            アップロードするファイルが同名のファイルを上書きするかどうか

        Returns
        -------
        File
            アップロードしたファイルの情報
        """
        file_byte = open(file, "rb") if file else None
        folder_id = self._folder_id or folder_id

        data = {
            "file": file_byte,
            "name": file_name,
            "folderId": folder_id,
            "comment": comment,
            "isSensitive": bool_to_string(is_sensitive),
            "force": bool_to_string(force),
        }
        res: IDriveFile = await self._session.request(
            Route("POST", "/api/drive/files/create"),
            data=data,
            auth=True,
            lower=True,
        )
        return File(res, client=self._client)


class ClientFolderActions(AbstractAction):
    def __init__(
        self, folder_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        self._folder_id = folder_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def create(self, name: str, parent_id: str | None = None) -> bool:
        """
        フォルダーを作成します

        Parameters
        ----------
        name : str, default=None
            フォルダーの名前
        parent_id : str | None, default=None
            親フォルダーのID

        Returns
        -------
        bool
            作成に成功したか否か
        """
        parent_id = parent_id or self._folder_id

        data = {"name": name, "parent_id": parent_id}
        res: bool = await self._session.request(
            Route("POST", "/api/drive/folders/create"),
            json=data,
            lower=True,
            auth=True,
        )
        return bool(res)

    async def delete(self, folder_id: str | None = None) -> bool:
        """
        Parameters
        ----------
        folder_id : str | None = None
            削除するノートのID

        Returns
        -------
        bool
            削除に成功したか否か
        """
        folder_id = folder_id or self._folder_id
        data = {"folderId": folder_id}
        res: bool = await self._session.request(
            Route("POST", "/api/drive/folders/delete"),
            json=data,
            lower=True,
            auth=True,
        )
        return bool(res)

    async def get_files(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        folder_id: str | None = None,
        file_type: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[File, None]:
        """
        ファイルを取得します

        Parameters
        ----------
        limit : int, default=10
            取得する上限
        since_id : str | None, default=None
            指定すると、そのIDを起点としてより新しいファイルを取得します
        until_id : str | None, default=None
            指定すると、そのIDを起点としてより古いファイルを取得します
        folder_id : str | None, default=None
            指定すると、そのフォルダーを起点としてファイルを取得します
        file_type : str | None, default=None
            取得したいファイルの拡張子
        """
        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        folder_id = self._folder_id or folder_id

        body = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "folderId": folder_id,
            "Type": file_type,
        }

        pagination = Pagination[IDriveFile](
            self._session, Route("POST", "/api/drive/files"), json=body
        )

        while True:
            res_drive_files = await pagination.next()

            for res_drive_file in res_drive_files:
                yield File(res_drive_file, client=self._client)

            if get_all is False or pagination.is_final:
                break


class FolderActions(ClientFolderActions):
    def __init__(
        self, folder_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        super().__init__(folder_id=folder_id, session=session, client=client)


class DriveActions(AbstractAction):
    def __init__(self, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def get_folders(
        self,
        limit: int = 100,
        since_id: str | None = None,
        until_id: str | None = None,
        folder_id: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[Folder, None]:
        """
        フォルダーの一覧を取得します

        Parameters
        ----------
        limit : int, default=10
            取得する上限
        since_id : str | None, default=None
            指定すると、その投稿を投稿を起点としてより新しい投稿を取得します
        until_id : str | None, default=None
            指定すると、その投稿を投稿を起点としてより古い投稿を取得します
        folder_id : str | None, default=None
            指定すると、そのフォルダーを起点としてフォルダーを取得します
        get_all : bool, default=False
            Whether to retrieve all folders or not
        """

        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        body = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "folderId": folder_id,
        }

        pagination = Pagination[FolderPayload](
            self._session, Route("POST", "/api/drive/folders"), json=body
        )

        while True:
            res_folders = await pagination.next()
            for res_folder in res_folders:
                yield Folder(res_folder, client=self._client)
            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.lite.meta import LiteMeta, MetaCommon
from mipac.types.meta import IAdminMeta, IFeatures, IMeta, IPolicies

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class Policies(AbstractModel):
    def __init__(self, policies: IPolicies) -> None:
        self.__policies: IPolicies = policies

    @property
    def gtl_available(self) -> bool:
        return self.__policies["gtl_available"]

    @property
    def ltl_available(self) -> bool:
        return self.__policies["ltl_available"]

    @property
    def can_public_note(self) -> bool:
        return self.__policies["can_public_note"]

    @property
    def can_invite(self) -> bool:
        return self.__policies["can_invite"]

    @property
    def can_manage_custom_emojis(self) -> bool:
        return self.__policies["can_manage_custom_emojis"]

    @property
    def can_hide_ads(self) -> bool:
        return self.__policies["can_hide_ads"]

    @property
    def drive_capacity_mb(self) -> int:
        return self.__policies["drive_capacity_mb"]

    @property
    def pin_limit(self) -> int:
        return self.__policies["pin_limit"]

    @property
    def antenna_limit(self) -> int:
        return self.__policies["antenna_limit"]

    @property
    def word_mute_limit(self) -> int:
        return self.__policies["word_mute_limit"]

    @property
    def webhook_limit(self) -> int:
        return self.__policies["webhook_limit"]

    @property
    def clip_limit(self) -> int:
        return self.__policies["clip_limit"]

    @property
    def note_each_clips_limit(self) -> int:
        return self.__policies["note_each_clips_limit"]

    @property
    def user_list_limit(self) -> int:
        return self.__policies["user_list_limit"]

    @property
    def user_each_user_lists_limit(self) -> int:
        return self.__policies["user_each_user_lists_limit"]

    @property
    def rate_limit_factor(self) -> int:
        return self.__policies["rate_limit_factor"]


class Features(AbstractModel):
    def __init__(self, features: IFeatures) -> None:
        self.__features = features

    @property
    def registration(self) -> bool:
        return self.__features["registration"]

    @property
    def elasticsearch(self) -> bool:
        return self.__features["elasticsearch"]

    @property
    def hcaptcha(self) -> bool:
        return self.__features["hcaptcha"]

    @property
    def recaptcha(self) -> bool:
        return self.__features["recaptcha"]

    @property
    def object_storage(self) -> bool:
        return self.__features["object_storage"]

    @property
    def twitter(self) -> bool:
        return self.__features["twitter"]

    @property
    def github(self) -> bool:
        return self.__features["github"]

    @property
    def discord(self) -> bool:
        return self.__features["discord"]

    @property
    def service_worker(self) -> bool:
        return self.__features["service_worker"]

    @property
    def miauth(self) -> bool:
        return self.__features.get("miauth", False)

    @property
    def email_required_for_signup(self) -> bool:
        return self.__features.get("email_required_for_signup", False)

    @property
    def global_time_line(self) -> bool:
        return self.__features.get("global_time_line", False)

    @property
    def local_time_line(self) -> bool:
        return self.__features.get("local_time_line", False)


class Meta(LiteMeta):
    def __init__(self, meta: IMeta, *, client: ClientManager) -> None:
        super().__init__(meta, client=client)
        self.__meta = meta

    # shared

    @property
    def features(self) -> Features:
        return Features(self.__meta["features"])

    # v12

    @property
    def pinned_clip_id(self) -> str | None:
        return self.__meta.get("pinned_clip_id")

    @property
    def pinned_pages(self) -> list[str]:
        return self.__meta.get("pinned_pages", [])

    @property
    def policies(self) -> Policies | None:
        return Policies(self.__meta["policies"]) if "policies" in self.__meta else None

    @property
    def require_setup(self) -> bool:
        return self.__meta.get("require_setup", False)


class AdminMeta(MetaCommon):
    def __init__(self, admin_meta: IAdminMeta, *, client: ClientManager) -> None:
        super().__init__(admin_meta, client=client)
        self.__admin_meta = admin_meta

    # 全てのバージョン共通に存在する

    @property
    def drive_capacity_per_local_user_mb(self) -> int | None:
        return self.__admin_meta.get("drive_capacity_per_local_user_mb")

    @property
    def drive_capacity_per_remote_user_mb(self) -> int | None:
        return self.__admin_meta.get("drive_capacity_per_remote_user_mb")

    @property
    def hidden_tags(self) -> list[str]:
        return self.__admin_meta.get("hidden_tags", [])

    @property
    def blocked_hosts(self) -> list[str]:
        return self.__admin_meta.get("blocked_hosts", [])

    @property
    def recaptcha_secret_key(self) -> str | None:
        return self.__admin_meta.get("recaptcha_secret_key")

    @property
    def twitter_consumer_key(self) -> str | None:
        return self.__admin_meta.get("twitter_consumer_key")

    @property
    def twitter_consumer_secret(self) -> str | None:
        return self.__admin_meta.get("twitter_consumer_secret")

    @property
    def github_client_id(self) -> str | None:
        return self.__admin_meta.get("github_client_id")

    @property
    def github_client_secret(self) -> str | None:
        return self.__admin_meta.get("github_client_secret")

    @property
    def discord_client_id(self) -> str | None:
        return self.__admin_meta.get("discord_client_id")

    @property
    def discord_client_secret(self) -> str | None:
        return self.__admin_meta.get("discord_client_secret")

    @property
    def email(self) -> str | None:
        return self.__admin_meta.get("email")

    @property
    def smtp_secure(self) -> bool:
        return self.__admin_meta.get("smtp_secure", False)

    @property
    def smtp_host(self) -> str | None:
        return self.__admin_meta.get("smtp_host")

    @property
    def smtp_port(self) -> int | None:
        return self.__admin_meta.get("smtp_port")

    @property
    def smtp_user(self) -> str | None:
        return self.__admin_meta.get("smtp_user")

    @property
    def smtp_pass(self) -> str | None:
        return self.__admin_meta.get("smtp_pass")

    @property
    def sw_private_key(self) -> str | None:
        return self.__admin_meta.get("sw_private_key")

    @property
    def use_object_storage(self) -> bool:
        return self.__admin_meta.get("use_object_storage", False)

    @property
    def object_storage_base_url(self) -> str | None:
        return self.__admin_meta.get("object_storage_base_url")

    @property
    def object_storage_bucket(self) -> str | None:
        return self.__admin_meta.get("object_storage_bucket")

    @property
    def object_storage_prefix(self) -> str | None:
        return self.__admin_meta.get("object_storage_prefix")

    @property
    def object_storage_endpoint(self) -> str | None:
        return self.__admin_meta.get("object_storage_endpoint")

    @property
    def object_storage_region(self) -> str | None:
        return self.__admin_meta.get("object_storage_region")

    @property
    def object_storage_port(self) -> int | None:
        return self.__admin_meta.get("object_storage_port")

    @property
    def object_storage_access_key(self) -> str | None:
        return self.__admin_meta.get("object_storage_access_key")

    @property
    def object_storage_secret_key(self) -> str | None:
        return self.__admin_meta.get("object_storage_secret_key")

    @property
    def object_storage_use_ssl(self) -> bool:
        return self.__admin_meta.get("object_storage_use_ssl", False)

    @property
    def object_storage_use_proxy(self) -> bool:
        return self.__admin_meta.get("object_storage_use_proxy", False)

    @property
    def object_storage_set_public_read(self) -> bool:
        return self.__admin_meta.get("object_storage_set_public_read", False)

    @property
    def pinned_users(self) -> list[str]:
        return self.__admin_meta.get("pinned_users", [])

    """
    v12 only
    """

    @property
    def hcaptcha_secret_key(self) -> str | None:
        return self.__admin_meta.get("hcaptcha_secret_key")

    @property
    def sensitive_media_detection(self) -> str | None:
        return self.__admin_meta.get("sensitive_media_detection")

    @property
    def sensitive_media_detection_sensitivity(self) -> str | None:
        return self.__admin_meta.get("sensitive_media_detection_sensitivity")

    @property
    def set_sensitive_flag_automatically(self) -> bool:
        return self.__admin_meta.get("set_sensitive_flag_automatically", False)

    @property
    def enable_sensitive_media_detection_for_videos(self) -> bool:
        return self.__admin_meta.get("enable_sensitive_media_detection_for_videos", False)

    @property
    def proxy_account_id(self) -> str | None:
        return self.__admin_meta.get("proxy_account_id")

    @property
    def summary_proxy(self) -> str | None:
        return self.__admin_meta.get("summary_proxy")

    @property
    def enable_ip_logging(self) -> bool:
        return self.__admin_meta.get("enable_ip_logging", False)

    @property
    def enable_active_email_validation(self) -> bool:
        return self.__admin_meta.get("enable_active_email_validation", False)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.moderator import AdminModeratorActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ("AdminModeratorManager",)


class AdminModeratorManager(AbstractManager):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str | None = user_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminModeratorActions:
        """Moderatorに関するアクション

        Returns
        -------
        AdminModeratorActions
            Moderatorに対するアクションを行うクラス
        """

        return AdminModeratorActions(session=self.__session, client=self.__client)



from __future__ import annotations

from typing import Any, TypedDict

__all__ = ("IFileProperties", "FolderPayload", "IDriveFile")


class IFileProperties(TypedDict):
    """
    プロパティー情報
    """

    width: int
    height: int
    avg_color: str | None


class FolderPayload(TypedDict):
    """
    フォルダーの情報
    """

    id: str
    created_at: str
    name: str
    folders_count: int
    files_count: int
    parent_id: str
    parent: dict[str, Any]


class IDriveFile(TypedDict):
    """
    ファイル情報
    """

    id: str
    created_at: str
    is_sensitive: bool
    name: str
    thumbnail_url: str
    url: str
    type: str
    size: int
    md5: str
    blurhash: str
    properties: IFileProperties



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.chart import ChartActions

if TYPE_CHECKING:
    from mipac.client import ClientManager
    from mipac.http import HTTPClient

__all__ = ("ChartManager",)


class ChartManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> ChartActions:
        return ChartActions(session=self.__session, client=self.__client)



from typing import Literal

IT_ACHIEVEMENT_NAME = Literal[
    "notes1",
    "notes10",
    "notes100",
    "notes500",
    "notes1000",
    "notes5000",
    "notes10000",
    "notes20000",
    "notes30000",
    "notes40000",
    "notes50000",
    "notes60000",
    "notes70000",
    "notes80000",
    "notes90000",
    "notes100000",
    "login3",
    "login7",
    "login15",
    "login30",
    "login60",
    "login100",
    "login200",
    "login300",
    "login400",
    "login500",
    "login600",
    "login700",
    "login800",
    "login900",
    "login1000",
    "passedSinceAccountCreated1",
    "passedSinceAccountCreated2",
    "passedSinceAccountCreated3",
    "loggedInOnBirthday",
    "loggedInOnNewYearsDay",
    "noteClipped1",
    "noteFavorited1",
    "myNoteFavorited1",
    "profileFilled",
    "markedAsCat",
    "following1",
    "following10",
    "following50",
    "following100",
    "following300",
    "followers1",
    "followers10",
    "followers50",
    "followers100",
    "followers300",
    "followers500",
    "followers1000",
    "collectAchievements30",
    "viewAchievements3min",
    "iLoveMisskey",
    "foundTreasure",
    "client30min",
    "client60min",
    "noteDeletedWithin1min",
    "postedAtLateNight",
    "postedAt0min0sec",
    "selfQuote",
    "htl20npm",
    "viewInstanceChart",
    "outputHelloWorldOnScratchpad",
    "open3windows",
    "driveFolderCircularReference",
    "reactWithoutRead",
    "clickedClickHere",
    "justPlainLucky",
    "setNameToSyuilo",
    "cookieClicked",
    "brainDiver",
]



from __future__ import annotations

from typing import TYPE_CHECKING, Any, AsyncGenerator, Generic, Literal, Type, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.http import HTTPClient, Route

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager

T = TypeVar("T")
M = TypeVar("M", bound=AbstractModel)


class Pagination(Generic[T]):
    def __init__(
        self,
        http_client: HTTPClient,
        route: Route,
        json: dict[str, Any],
        auth: bool = True,
        remove_none: bool = True,
        lower: bool = True,
        pagination_type: Literal["until", "count"] = "until",
        can_use_limit: bool = True,
        limit: int = 100,
        max_limit: int = 100,
    ) -> None:
        self.http_client: HTTPClient = http_client
        self.route: Route = route
        self.json: dict[str, Any] = json
        self.auth: bool = auth
        self.remove_none: bool = remove_none
        self.lower: bool = lower
        self.pagination_type: Literal["until", "count"] = pagination_type
        self.can_use_limit: bool = can_use_limit
        self.limit: int = limit
        self.max_limit: int = max_limit
        self.count = 0
        self.next_id: str = ""
        self.previous_id: str = ""
        self.latest_res_count: int = 0

    async def next(self) -> list[T]:
        if self.pagination_type == "count":
            self.json["offset"] = self.json.get("limit", self.limit) * self.count
            self.count += 1
        res: list[T] = await self.http_client.request(
            self.route,
            auth=self.auth,
            remove_none=self.remove_none,
            lower=self.lower,
            json=self.json,
        )
        if self.pagination_type == "until":
            self.previous_id = self.json.get("untilId", "")  # 前のIDを保存しておく
            if len(res) > 0:
                self.next_id = res[-1]["id"]  # type: ignore
            self.json["untilId"] = self.next_id
        self.latest_res_count = len(res)
        return res

    @property
    def is_final(self) -> bool:
        if (
            self.pagination_type == "count"
            and self.latest_res_count == 0
            or self.latest_res_count < self.max_limit
        ):
            return True
        if self.pagination_type == "until" and self.latest_res_count == 0:
            return True
        return False


async def pagination_iterator(
    pagination: Pagination, get_all: bool = False, *, model: Type[M], client: ClientManager
) -> AsyncGenerator[M, None]:
    while True:
        res = await pagination.next()
        for i in res:
            yield model(i, client=client)
        if pagination.is_final or get_all is False:
            break



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.lite.user import LiteUser
from mipac.models.note import Note

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.follow import FollowManager, FollowRequestManager
    from mipac.types.notification import (
        IAchievementNf,
        INoteNf,
        INotification,
        IPollEndNf,
        IReactionNf,
        IUserNf,
    )


class Notification(AbstractModel):
    def __init__(
        self,
        notification: INotification,
        *,
        client: ClientManager,
    ) -> None:
        self.__notification: INotification = notification
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__notification["id"]

    @property
    def type(self) -> str:
        return self.__notification["type"]

    @property
    def created_at(self) -> datetime:
        return datetime.strptime(self.__notification["created_at"], "%Y-%m-%dT%H:%M:%S.%fZ")

    @property
    def is_read(self) -> bool:
        return self.__notification["is_read"]

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Notification) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class NotificationFollow(Notification):
    def __init__(
        self,
        notification: IUserNf,
        *,
        client: ClientManager,
    ) -> None:
        super().__init__(notification, client=client)
        self.__notification: IUserNf = notification
        self.__client: ClientManager = client

    @property
    def user(self) -> LiteUser:
        return LiteUser(
            self.__notification["user"],
            client=self.__client,
        )

    @property
    def user_id(self) -> str:
        return self.__notification["user_id"]

    @property
    def api(self) -> FollowManager:
        return self.__client._create_user_instance(user=self.user).follow


class NotificationFollowRequest(Notification):
    def __init__(
        self,
        notification: IUserNf,
        *,
        client: ClientManager,
    ) -> None:
        super().__init__(notification, client=client)
        self.__notification: IUserNf = notification
        self.__client: ClientManager = client

    @property
    def user(self) -> LiteUser:
        return LiteUser(
            self.__notification["user"],
            client=self.__client,
        )

    @property
    def user_id(self) -> str:
        return self.__notification["user_id"]

    @property
    def api(self) -> FollowRequestManager:
        return self.__client._create_user_instance(user=self.user).follow.request


class NotificationNote(Notification):
    def __init__(
        self,
        notification: INoteNf,
        *,
        client: ClientManager,
    ) -> None:
        super().__init__(notification, client=client)
        self.__notification: INoteNf = notification
        self.__client: ClientManager = client

    @property
    def user(self) -> LiteUser:
        return LiteUser(
            self.__notification["user"],
            client=self.__client,
        )

    @property
    def user_id(self) -> str:
        return self.__notification["user_id"]

    @property
    def note(self) -> Note:
        return Note(
            self.__notification["note"],
            client=self.__client,
        )


class NotificationPollEnd(Notification):
    def __init__(
        self,
        notification: IPollEndNf,
        *,
        client: ClientManager,
    ) -> None:
        super().__init__(notification, client=client)
        self.__notification: IPollEndNf = notification
        self.__client: ClientManager = client

    @property
    def note(self) -> Note:
        return Note(
            self.__notification["note"],
            client=self.__client,
        )


class NotificationReaction(Notification):
    def __init__(self, reaction: IReactionNf, *, client: ClientManager) -> None:
        super().__init__(reaction, client=client)
        self.__notification: IReactionNf = reaction
        self.__client: ClientManager = client

    @property
    def user(self) -> LiteUser:
        return LiteUser(self.__notification["user"], client=self.__client)

    @property
    def note(self) -> Note:
        return Note(self.__notification["note"], client=self.__client)

    @property
    def reaction(self) -> str:
        return self.__notification["reaction"]


class NotificationAchievement(Notification):
    def __init__(
        self,
        notification: IAchievementNf,
        *,
        client: ClientManager,
    ) -> None:
        super().__init__(notification, client=client)
        self.__notification: IAchievementNf = notification
        self.__client: ClientManager = client

    @property
    def achievement(self) -> str:
        return self.__notification["achievement"]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.http import HTTPClient, Route

if TYPE_CHECKING:
    from mipac.client import ClientManager


class PagesManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def get_pages(
        self,
        limit: int = 100,
        since_id: int | None = None,
        until_id: int | None = None,
    ):
        data = {"limit": limit, "since_id": since_id, "until_id": until_id}
        res = await self.__session.request(Route("POST", "/api/i/pages"), json=data, auth=True)
        return res



from typing import Literal, TypedDict

IAntennaReceiveSource = Literal["home", "all", "users", "list"]


class IAntenna(TypedDict):
    case_sensitive: bool
    created_at: str
    exclude_keywords: list[str]
    has_unread_note: bool
    id: str
    is_actor: bool
    keywords: list[str]
    name: str
    notify: bool
    src: IAntennaReceiveSource
    user_list_id: str | None
    users: list[str]
    with_file: bool
    with_replies: bool



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Literal

from mipac.abstract.model import AbstractModel
from mipac.types.ads import IAd
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.admins.ad import AdminAdvertisingModelManager
    from mipac.manager.client import ClientManager


class Ad(AbstractModel):
    def __init__(self, ad_data: IAd, *, client: ClientManager) -> None:
        self.__ad_data: IAd = ad_data
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__ad_data["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__ad_data["created_at"])

    @property
    def starts_at(self) -> datetime:
        return str_to_datetime(self.__ad_data["id"])

    @property
    def expires_at(self) -> datetime:
        return str_to_datetime(self.__ad_data["id"])

    @property
    def url(self) -> str:
        return self.__ad_data["url"]

    @property
    def place(self) -> Literal["square" "horizontal" "horizontal-big"]:
        return self.__ad_data["place"]

    @property
    def priority(self) -> Literal["high" "middle" "low"]:
        return self.__ad_data["priority"]

    @property
    def ratio(self) -> int:
        return self.__ad_data["ratio"]

    @property
    def image_url(self) -> str:
        return self.__ad_data["image_url"]

    @property
    def memo(self) -> str | None:
        return self.__ad_data["memo"]

    @property
    def api(self) -> AdminAdvertisingModelManager:
        return self.__client.admin.create_ad_model_manager(ad_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Ad) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


__all__ = ("AbstractModel",)


class AbstractModel(ABC):
    @abstractmethod
    def __init__(self, data: Any, *, client: ClientManager) -> None:
        pass



from __future__ import annotations

from typing import TypedDict


class IFederationInstanceStat(TypedDict):
    top_sub_instances: list[IFederationInstance]
    other_followers_count: int
    top_pub_instances: list[IFederationInstance]
    other_following_count: int


class IFederationInstanceRequired(TypedDict):
    id: str
    host: str
    users_count: int
    notes_count: int
    following_count: int
    followers_count: int
    is_not_responding: bool
    is_suspended: bool
    is_blocked: bool
    software_name: str
    software_version: str
    open_registrations: bool
    name: str
    description: str
    maintainer_name: str
    maintainer_email: str
    icon_url: str
    favicon_url: str
    theme_color: str
    info_updated_at: str


class IFederationInstance(IFederationInstanceRequired, total=False):
    caught_at: str
    first_retrieved_at: str
    latest_request_sent_at: str
    last_communicated_at: str


class IInstanceLite(TypedDict):
    name: str
    software_name: str
    software_version: str
    icon_url: str
    favicon_url: str
    theme_color: str



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.user import MeDetailed, UserDetailed
from mipac.types.roles import IMeRole, IRole, IRolePolicies, IRolePolicieValue, IRoleUser
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.admins.roles import AdminRolesModelManager
    from mipac.manager.client import ClientManager
    from mipac.manager.user import UserManager


class RoleUser(AbstractModel):
    def __init__(self, role_user: IRoleUser, *, client: ClientManager) -> None:
        self.__role_user = role_user
        self.__client = client

    @property
    def id(self) -> str:
        return self.__role_user["id"]

    @property
    def user(self) -> UserDetailed:
        return UserDetailed(self.__role_user["user"], client=self.__client)

    @property
    def expires_at(self) -> datetime | None:
        return (
            str_to_datetime(self.__role_user["expires_at"])
            if self.__role_user["expires_at"]
            else None
        )

    @property
    def action(self) -> UserManager:
        return self.__client._create_user_instance(self.user)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, RoleUser) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class MeRole(AbstractModel):
    def __init__(self, data: IMeRole, *, client: ClientManager) -> None:
        self.__role_user = data
        self.__client = client

    @property
    def id(self) -> str:
        return self.__role_user["id"]

    @property
    def user(self) -> MeDetailed:
        return MeDetailed(self.__role_user["user"], client=self.__client)

    @property
    def expires_at(self) -> datetime | None:
        return (
            str_to_datetime(self.__role_user["expires_at"])
            if self.__role_user["expires_at"]
            else None
        )

    @property
    def action(self) -> UserManager:
        return self.__client._create_user_instance(self.user)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, RoleUser) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class RolePolicyValue(AbstractModel):
    def __init__(self, policiy_value_data: IRolePolicieValue) -> None:
        self.__policy_value_data = policiy_value_data

    @property
    def value(self) -> int:
        return self.__policy_value_data.get("value")

    @property
    def use_default(self) -> bool:
        return self.__policy_value_data.get("use_default")

    @property
    def priority(self) -> int | None:
        return self.__policy_value_data.get("priority")


class RolePolicies(AbstractModel):
    def __init__(self, role_policies_data: IRolePolicies) -> None:
        self.__role_policies_data = role_policies_data

    @property
    def antenna_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("antenna_limit"))

    @property
    def gtl_available(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("gtl_available"))

    @property
    def ltl_available(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("ltl_available"))

    @property
    def can_public_note(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("can_public_note"))

    @property
    def drive_capacity_mb(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("drive_capacity_mb"))

    @property
    def can_invite(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("can_invite"))

    @property
    def can_manage_custom_emojis(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("can_manage_custom_emojis"))

    @property
    def can_hide_ads(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("can_hide_ads"))

    @property
    def pin_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("pin_limit"))

    @property
    def word_mute_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("word_mute_limit"))

    @property
    def webhook_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("webhook_limit"))

    @property
    def clip_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("clip_limit"))

    @property
    def note_each_clips_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("note_each_clips_limit"))

    @property
    def user_list_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("user_list_limit"))

    @property
    def user_each_user_lists_limit(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("user_each_user_lists_limit"))

    @property
    def rate_limit_factor(self) -> RolePolicyValue:
        return RolePolicyValue(self.__role_policies_data.get("rate_limit_factor"))


class Role(AbstractModel):
    def __init__(self, role_data: IRole, *, client: ClientManager) -> None:
        self.__role_data: IRole = role_data
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__role_data.get("id")

    @property
    def created_at(self) -> str:
        return self.__role_data.get("created_at")

    @property
    def updated_at(self) -> str:
        return self.__role_data.get("updated_at")

    @property
    def name(self) -> str:
        return self.__role_data.get("name")

    @property
    def description(self) -> str:
        return self.__role_data.get("description")

    @property
    def color(self) -> str | None:
        return self.__role_data.get("color")

    @property
    def icon_url(self) -> str | None:
        return self.__role_data.get("icon_url")

    @property
    def target(self) -> str:
        return self.__role_data.get("target")

    @property
    def cond_formula(self) -> dict:
        return self.__role_data.get("cond_formula")

    @property
    def is_public(self) -> bool:
        return self.__role_data.get("is_public")

    @property
    def is_administrator(self) -> bool:
        return self.__role_data.get("is_administrator")

    @property
    def is_moderator(self) -> bool:
        return self.__role_data.get("is_moderator")

    @property
    def as_badge(self) -> bool:
        return self.__role_data.get("as_badge")

    @property
    def can_edit_members_by_moderator(self) -> bool:
        return self.__role_data.get("can_edit_members_by_moderator")

    @property
    def policies(self) -> RolePolicies:
        return RolePolicies(self.__role_data.get("policies"))

    @property
    def users_count(self) -> int:
        return self.__role_data.get("users_count")

    @property
    def api(self) -> AdminRolesModelManager:
        return self.__client.admin.create_roles_model_manager(self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Role) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.reaction import ReactionActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager


class ReactionManager(AbstractManager):
    def __init__(self, note_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__note_id: str | None = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> ReactionActions:
        """リアクションに関するアクション

        Returns
        -------
        ReactionActions
            Reactionに対するアクションを行うクラス
        """
        return ReactionActions(
            note_id=self.__note_id,
            session=self.__session,
            client=self.__client,
        )



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Generic, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.models.drive import File
from mipac.models.lite.user import LiteUser
from mipac.types.note import INoteVisibility, IPartialNote
from mipac.types.reaction import IReactionAcceptance
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.note import ClientNoteManager


T = TypeVar("T", bound=IPartialNote)


class PartialNote(Generic[T], AbstractModel):
    def __init__(self, note_data: T, client: ClientManager) -> None:
        self._note: T = note_data
        self._client: ClientManager = client

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self._note["created_at"])

    @property
    def cw(self) -> str | None:
        return self._note.get("cw")

    @property
    def file_ids(self) -> list[str]:
        return self._note["file_ids"]

    @property
    def files(self) -> list[File]:
        return [File(file, client=self._client) for file in self._note["files"]]

    @property
    def id(self) -> str:
        """
        ノートのID

        Returns
        -------
        str
            ユーザーのID
        """
        return self._note["id"]

    @property
    def reaction_acceptance(self) -> IReactionAcceptance | None:
        """リアクションを受け入れ

        Returns
        -------
        IReactionAcceptance | None
        """
        return self._note.get("reaction_acceptance")

    @property
    def reaction_emojis(self) -> dict[str, str] | None:
        """リアクション一覧です

        Returns
        -------
        dict[str, str] | None
            リアクション名がキー、値はリアクション画像のリンクです
        """
        return self._note.get("reaction_emojis")

    @property
    def renote_id(self) -> str | None:
        return self._note["renote_id"]

    @property
    def renote_count(self) -> int:
        return self._note["renote_count"]

    @property
    def reactions(self) -> dict[str, int]:
        return self._note["reactions"]

    @property
    def replies_count(self) -> int:
        return self._note["replies_count"]

    @property
    def reply_id(self) -> str | None:
        return self._note["reply_id"]

    @property
    def tags(self) -> list[str]:
        return self._note.get("tags", [])

    @property
    def content(self) -> str | None:
        return self._note.get("text")

    @property
    def author(self) -> LiteUser:
        return LiteUser(self._note["user"], client=self._client)

    @property
    def user_id(self) -> str:
        return self._note["user_id"]

    @property
    def visibility(
        self,
    ) -> INoteVisibility:
        return self._note["visibility"]

    @property
    def api(self) -> ClientNoteManager:
        """
        ノートに対するアクション

        Returns
        -------
        NoteActions
        """
        return self._client.note.create_client_note_manager(self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, PartialNote) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)
